# encoding: utf-8

'''Unit tests for ckan/logic/auth/update.py.

'''
import mock
import nose

import ckan.logic as logic
import ckan.model as model
import ckan.plugins as p

import ckan.tests.helpers as helpers
import ckan.tests.factories as factories


assert_equals = nose.tools.assert_equals
assert_raises = nose.tools.assert_raises


class TestUpdate(object):

    def test_user_update_visitor_cannot_update_user(self):
        '''Visitors should not be able to update users' accounts.'''

        # Make a mock ckan.model.User object, Fred.
        fred = factories.MockUser(name='fred')

        # Make a mock ckan.model object.
        mock_model = mock.MagicMock()
        # model.User.get(user_id) should return Fred.
        mock_model.User.get.return_value = fred

        # Put the mock model in the context.
        # This is easier than patching import ckan.model.
        context = {'model': mock_model}

        # No user is going to be logged-in.
        context['user'] = '127.0.0.1'

        # Make the visitor try to update Fred's user account.
        params = {
            'id': fred.id,
            'name': 'updated_user_name',
        }

        nose.tools.assert_raises(logic.NotAuthorized, helpers.call_auth,
                                 'user_update', context=context, **params)

    ## START-AFTER

    def test_user_update_user_cannot_update_another_user(self):
        '''Users should not be able to update other users' accounts.'''

        # 1. Setup.

        # Make a mock ckan.model.User object, Fred.
        fred = factories.MockUser(name='fred')

        # Make a mock ckan.model object.
        mock_model = mock.MagicMock()
        # model.User.get(user_id) should return Fred.
        mock_model.User.get.return_value = fred

        # Put the mock model in the context.
        # This is easier than patching import ckan.model.
        context = {'model': mock_model}

        # The logged-in user is going to be Bob, not Fred.
        context['user'] = 'bob'

        # 2. Call the function that's being tested, once only.

        # Make Bob try to update Fred's user account.
        params = {
            'id': fred.id,
            'name': 'updated_user_name',
        }

        # 3. Make assertions about the return value and/or side-effects.

        nose.tools.assert_raises(logic.NotAuthorized, helpers.call_auth,
                                 'user_update', context=context, **params)

        # 4. Do nothing else!

    ## END-BEFORE

    def test_user_update_user_can_update_herself(self):
        '''Users should be authorized to update their own accounts.'''

        # Make a mock ckan.model.User object, Fred.
        fred = factories.MockUser(name='fred')

        # Make a mock ckan.model object.
        mock_model = mock.MagicMock()
        # model.User.get(user_id) should return our mock user.
        mock_model.User.get.return_value = fred

        # Put the mock model in the context.
        # This is easier than patching import ckan.model.
        context = {'model': mock_model}

        # The 'user' in the context has to match fred.name, so that the
        # auth function thinks that the user being updated is the same user as
        # the user who is logged-in.
        context['user'] = fred.name

        # Make Fred try to update his own user name.
        params = {
            'id': fred.id,
            'name': 'updated_user_name',
        }

        result = helpers.call_auth('user_update', context=context, **params)
        assert result is True

    def test_user_update_with_no_user_in_context(self):

        # Make a mock ckan.model.User object.
        mock_user = factories.MockUser(name='fred')

        # Make a mock ckan.model object.
        mock_model = mock.MagicMock()
        # model.User.get(user_id) should return our mock user.
        mock_model.User.get.return_value = mock_user

        # Put the mock model in the context.
        # This is easier than patching import ckan.model.
        context = {'model': mock_model}

        # For this test we're going to have no 'user' in the context.
        context['user'] = None

        params = {
            'id': mock_user.id,
            'name': 'updated_user_name',
        }

        nose.tools.assert_raises(logic.NotAuthorized, helpers.call_auth,
                                 'user_update', context=context, **params)

    def test_user_generate_own_apikey(self):
        fred = factories.MockUser(name='fred')
        mock_model = mock.MagicMock()
        mock_model.User.get.return_value = fred
        # auth_user_obj shows user as logged in for non-anonymous auth
        # functions
        context = {'model': mock_model, 'auth_user_obj': fred}
        context['user'] = fred.name
        params = {
            'id': fred.id,
        }

        result = helpers.call_auth('user_generate_apikey', context=context,
                                   **params)
        assert result is True

    def test_user_generate_apikey_without_logged_in_user(self):
        fred = factories.MockUser(name='fred')
        mock_model = mock.MagicMock()
        mock_model.User.get.return_value = fred
        context = {'model': mock_model}
        context['user'] = None
        params = {
            'id': fred.id,
        }

        nose.tools.assert_raises(logic.NotAuthorized, helpers.call_auth,
                                 'user_generate_apikey', context=context,
                                 **params)

    def test_user_generate_apikey_for_another_user(self):
        fred = factories.MockUser(name='fred')
        bob = factories.MockUser(name='bob')
        mock_model = mock.MagicMock()
        mock_model.User.get.return_value = fred
        # auth_user_obj shows user as logged in for non-anonymous auth
        # functions
        context = {'model': mock_model, 'auth_user_obj': bob}
        context['user'] = bob.name
        params = {
            'id': fred.id,
        }

        nose.tools.assert_raises(logic.NotAuthorized, helpers.call_auth,
                                 'user_generate_apikey', context=context,
                                 **params)


class TestUpdateResourceViews(object):

    @classmethod
    def setup_class(cls):
        if not p.plugin_loaded('image_view'):
            p.load('image_view')

        helpers.reset_db()

    @classmethod
    def teardown_class(cls):
        p.unload('image_view')

    def test_anon_can_not_update(self):

        resource_view = factories.ResourceView()

        params = {'id': resource_view['id'],
                  'title': 'Resource View Updated',
                  'view_type': 'image_view',
                  'image_url': 'url'}

        context = {'user': None, 'model': model}
        nose.tools.assert_raises(logic.NotAuthorized, helpers.call_auth,
                                 'resource_view_update', context=context,
                                 **params)

    def test_authorized_if_user_has_permissions_on_dataset(self):

        user = factories.User()

        dataset = factories.Dataset(user=user)

        resource = factories.Resource(user=user, package_id=dataset['id'])

        resource_view = factories.ResourceView(resource_id=resource['id'])

        params = {'id': resource_view['id'],
                  'resource_id': resource['id'],
                  'title': 'Resource View Updated',
                  'view_type': 'image_view',
                  'image_url': 'url'}

        context = {'user': user['name'], 'model': model}
        response = helpers.call_auth('resource_view_update', context=context,
                                     **params)

        assert_equals(response, True)

    def test_not_authorized_if_user_has_no_permissions_on_dataset(self):

        org = factories.Organization()

        user = factories.User()

        member = {'username': user['name'],
                  'role': 'admin',
                  'id': org['id']}
        helpers.call_action('organization_member_create', **member)

        user_2 = factories.User()

        dataset = factories.Dataset(owner_org=org['id'])

        resource = factories.Resource(package_id=dataset['id'])

        resource_view = factories.ResourceView(resource_id=resource['id'])

        params = {'id': resource_view['id'],
                  'resource_id': resource['id'],
                  'title': 'Resource View Updated',
                  'view_type': 'image_view',
                  'image_url': 'url'}

        context = {'user': user_2['name'], 'model': model}
        nose.tools.assert_raises(logic.NotAuthorized, helpers.call_auth,
                                 'resource_view_update', context=context,
                                 **params)


class TestConfigOptionUpdateAuth(object):

    def setup(self):
        helpers.reset_db()

    def test_config_option_update_anon_user(self):
        '''An anon user is not authorized to use config_option_update
        action.'''
        context = {'user': None, 'model': None}
        assert_raises(logic.NotAuthorized, helpers.call_auth,
                      'config_option_update', context=context)

    def test_config_option_update_normal_user(self):
        '''A normal logged in user is not authorized to use config_option_update
        action.'''
        factories.User(name='fred')
        context = {'user': 'fred', 'model': None}
        assert_raises(logic.NotAuthorized, helpers.call_auth,
                      'config_option_update', context=context)

    def test_config_option_update_sysadmin(self):
        '''A sysadmin is authorized to use config_option_update action.'''
        factories.Sysadmin(name='fred')
        context = {'user': 'fred', 'model': None}
        assert helpers.call_auth('config_option_update', context=context)



# encoding: utf-8

import ckan
import pylons.test
import paste.fixture
import ckan.lib.helpers as helpers
import ckan.lib.dictization.model_dictize as model_dictize


class TestVocabulary(object):

    @classmethod
    def setup_class(self):
        self.app = paste.fixture.TestApp(pylons.test.pylonsapp)

    @classmethod
    def teardown_class(self):
        ckan.model.repo.rebuild_db()

    def setup(self):
        self.clean_vocab()
        model = ckan.model
        context = {'model': model}

        genre = model.Vocabulary("Genre")
        time_period = ckan.model.Vocabulary("Time Period")
        composers = ckan.model.Vocabulary("Composers")
        model.Session.add_all([genre, time_period, composers])

        self.genre_vocab = model_dictize.vocabulary_dictize(genre, context)
        self.timeperiod_vocab = model_dictize.vocabulary_dictize(time_period,
                context)
        self.composers_vocab = model_dictize.vocabulary_dictize(composers,
                context)
        ckan.model.Session.commit()

        self.sysadmin_user = ckan.model.User.get('admin')
        self.normal_user = ckan.model.User.get('normal')
        if not self.sysadmin_user:
            normal_user = ckan.model.User(name=u'normal', password=u'annafan')
            sysadmin_user = ckan.model.User(name=u'admin',
                    password=u'testsysadmin')
            sysadmin_user.sysadmin = True
            ckan.model.Session.add(normal_user)
            ckan.model.Session.add(sysadmin_user)
            ckan.model.Session.commit()
            self.sysadmin_user = ckan.model.User.get('admin')
            self.normal_user = ckan.model.User.get('normal')
        self.sysadmin_apikey = self.sysadmin_user.apikey

    def clean_vocab(self):
        ckan.model.Session.execute('delete from package_tag_revision')
        ckan.model.Session.execute('delete from package_tag')
        ckan.model.Session.execute('delete from tag')
        ckan.model.Session.execute('delete from vocabulary')
        ckan.model.Session.commit()

    @classmethod
    def _post(self, url, params=None, extra_environ=None):
        if params is None:
            params = {}
        param_string = helpers.json.dumps(params)
        response = self.app.post(url, params=param_string,
                extra_environ=extra_environ)
        assert not response.errors
        return response.json

    @classmethod
    def _create_vocabulary(self, vocab_name=None, user=None):
        # Create a new vocabulary.
        params = {'name': vocab_name}
        if user:
            extra_environ = {'Authorization': str(user.apikey)}
        else:
            extra_environ = None
        response = self._post('/api/action/vocabulary_create', params=params,
                extra_environ=extra_environ)

        # Check the values of the response.
        assert response['success'] is True
        assert response['result']
        created_vocab = response['result']
        assert created_vocab['name'] == vocab_name
        assert created_vocab['id']

        # Get the list of vocabularies.
        response = self._post('/api/action/vocabulary_list')
        # Check that the vocabulary we created is in the list.
        assert response['success'] is True
        assert response['result']
        assert response['result'].count(created_vocab) == 1

        # Get the created vocabulary.
        params = {'id': created_vocab['id']}
        response = self._post('/api/action/vocabulary_show', params)
        # Check that retrieving the vocab by name gives the same result.
        by_name_params = {'id': created_vocab['name']}
        assert response == self._post('/api/action/vocabulary_show',
                by_name_params)
        # Check that it matches what we created.
        assert response['success'] is True
        assert response['result'] == created_vocab

        return created_vocab

    def _update_vocabulary(self, params, user=None):
        if user:
            extra_environ = {'Authorization': str(user.apikey)}
        else:
            extra_environ = None

        original_vocab = self._post('/api/action/vocabulary_show',
                {'id': params.get('id') or params.get('name')})['result']

        response = self._post('/api/action/vocabulary_update', params=params,
                extra_environ=extra_environ)

        # Check the values of the response.
        assert response['success'] is True
        assert response['result']
        updated_vocab = response['result']
        # id should never change.
        assert updated_vocab['id'] == original_vocab['id']
        if 'id' in params:
            assert updated_vocab['id'] == params['id']
        # name should change only if given in params.
        if 'name' in params:
            assert updated_vocab['name'] == params['name']
        else:
            assert updated_vocab['name'] == original_vocab['name']
        # tags should change only if given in params.
        if 'tags' in params:
            assert sorted([tag['name'] for tag in params['tags']]) \
                    == sorted([tag['name'] for tag in updated_vocab['tags']])
        else:
            assert updated_vocab['tags'] == original_vocab['tags']

        # Get the list of vocabularies.
        response = self._post('/api/action/vocabulary_list')
        # Check that the vocabulary we created is in the list.
        assert response['success'] is True
        assert response['result']
        assert response['result'].count(updated_vocab) == 1

        # Get the created vocabulary.
        params = {'id': updated_vocab['id']}
        response = self._post('/api/action/vocabulary_show', params)
        # Check that retrieving the vocab by name gives the same result.
        by_name_params = {'id': updated_vocab['name']}
        assert response == self._post('/api/action/vocabulary_show',
                by_name_params)
        # Check that it matches what we created.
        assert response['success'] is True
        assert response['result'] == updated_vocab

        return updated_vocab

    def _delete_vocabulary(self, vocab_id, user=None):
        if user:
            extra_environ = {'Authorization': str(user.apikey)}
        else:
            extra_environ = None
        params = {'id': vocab_id}
        response = self._post('/api/action/vocabulary_delete', params=params,
                extra_environ=extra_environ)

        # Check the values of the response.
        assert response['success'] is True
        assert response['result'] is None
        response['result']

        # Get the list of vocabularies.
        response = self._post('/api/action/vocabulary_list')
        assert response['success'] is True
        assert response['result']
        # Check that the vocabulary we deleted is not in the list.
        assert vocab_id not in [vocab['id'] for vocab in response['result']]

        # Check that the deleted vocabulary can no longer be retrieved.
        response = self.app.post('/api/action/vocabulary_show',
                params=helpers.json.dumps(params),
                extra_environ={'Authorization':
                    str(self.sysadmin_user.apikey)},
                status=404)
        assert response.json['success'] is False

    def _list_tags(self, vocabulary=None, user=None):
        params = {}
        if vocabulary:
            params['vocabulary_id'] = vocabulary['id']
        if user:
            extra_environ = {'Authorization': str(user.apikey)}
        else:
            extra_environ = None
        response = self._post('/api/action/tag_list', params=params,
                extra_environ=extra_environ)
        assert response['success'] is True
        return response['result']

    def _create_tag(self, user, tag_name, vocabulary=None):
        tag_dict = {'name': tag_name}
        if vocabulary:
            tag_dict['vocabulary_id'] = vocabulary['id']
        if user:
            extra_environ = {'Authorization': str(user.apikey)}
        else:
            extra_environ = None
        response = self._post('/api/action/tag_create', params=tag_dict,
                extra_environ=extra_environ)
        assert response['success'] is True
        return response['result']

    def _delete_tag(self, user, tag_id_or_name, vocab_id_or_name=None):
        params = {'id': tag_id_or_name}
        if vocab_id_or_name:
            params['vocabulary_id'] = vocab_id_or_name
        if user:
            extra_environ = {'Authorization': str(user.apikey)}
        else:
            extra_environ = None
        response = self._post('/api/action/tag_delete', params=params,
                extra_environ=extra_environ)
        assert response['success'] is True
        return response['result']

    def test_vocabulary_create(self):
        '''Test adding a new vocabulary to a CKAN instance via the action
        API.

        '''
        self._create_vocabulary(vocab_name="My cool vocab",
                user=self.sysadmin_user)

    def test_vocabulary_create_with_tags(self):
        '''Test adding a new vocabulary with some tags.

        '''
        params = {'name': 'foobar'}
        tag1 = {'name': 'foo'}
        tag2 = {'name': 'bar'}
        params['tags'] = [tag1, tag2]
        response = self._post('/api/action/vocabulary_create',
                params=params,
                extra_environ={'Authorization': str(self.sysadmin_apikey)})
        assert response['success'] is True
        assert response['result']
        created_vocab = response['result']
        assert created_vocab['name'] == 'foobar'
        assert created_vocab['id']

        # Get the list of vocabularies.
        response = self._post('/api/action/vocabulary_list')
        # Check that the vocabulary we created is in the list.
        assert response['success'] is True
        assert response['result']
        assert response['result'].count(created_vocab) == 1

        # Get the created vocabulary.
        params = {'id': created_vocab['id']}
        response = self._post('/api/action/vocabulary_show', params)
        # Check that retrieving the vocab by name gives the same result.
        by_name_params = {'id': created_vocab['name']}
        assert response == self._post('/api/action/vocabulary_show',
                by_name_params)
        # Check that it matches what we created.
        assert response['success'] is True
        assert response['result'] == created_vocab

        # Get the list of tags for the vocabulary.
        tags = self._list_tags(created_vocab)
        assert len(tags) == 2
        assert tags.count('foo') == 1
        assert tags.count('bar') == 1

    def test_vocabulary_create_bad_tags(self):
        '''Test creating new vocabularies with invalid tags.

        '''
        for tags in (
                [{'id': 'xxx'}, {'name': 'foo'}],
                [{'name': 'foo'}, {'name': None}],
                [{'name': 'foo'}, {'name': ''}],
                [{'name': 'foo'}, {'name': 'f'}],
                [{'name': 'f' * 200}, {'name': 'foo'}],
                [{'name': 'Invalid!'}, {'name': 'foo'}],
                ):
            params = {'name': 'foobar', 'tags': tags}
            response = self.app.post('/api/action/vocabulary_create',
                    params=helpers.json.dumps(params),
                    extra_environ={'Authorization': str(self.sysadmin_apikey)},
                    status=409)
            assert response.json['success'] is False
            assert 'tags' in response.json['error']
            assert len(response.json['error']) == 2

    def test_vocabulary_create_none_tags(self):
        '''Test creating new vocabularies with None for 'tags'.

        '''
        params = {'name': 'foobar', 'tags': None}
        response = self.app.post('/api/action/vocabulary_create',
                params=helpers.json.dumps(params),
                extra_environ={'Authorization':
                    str(self.sysadmin_user.apikey)},
                status=400)
        assert "Integrity Error" in response.body

    def test_vocabulary_create_empty_tags(self):
        '''Test creating new vocabularies with [] for 'tags'.

        '''
        params = {'name': 'foobar', 'tags': []}
        response = self.app.post('/api/action/vocabulary_create',
                params=helpers.json.dumps(params),
                extra_environ={'Authorization':
                    str(self.sysadmin_user.apikey)},
                status=200)
        assert response.json['success'] is True
        assert response.json['result']
        created_vocab = response.json['result']
        assert created_vocab['name'] == 'foobar'
        assert created_vocab['id']
        assert created_vocab['tags'] == []
        params = {'id': created_vocab['id']}
        response = self._post('/api/action/vocabulary_show', params)
        assert response['success'] is True
        assert response['result'] == created_vocab
        tags = self._list_tags(created_vocab)
        assert tags == []

    def test_vocabulary_create_id(self):
        '''Test error response when user tries to supply their own ID when
        creating a vocabulary.

        '''
        params = {'id': 'xxx', 'name': 'foobar'}
        param_string = helpers.json.dumps(params)
        response = self.app.post('/api/action/vocabulary_create',
                params=param_string,
                extra_environ={'Authorization':
                    str(self.sysadmin_user.apikey)},
                status=409)
        assert response.json['success'] is False
        assert response.json['error']['id'] == [u'The input field id was '
            'not expected.']

    def test_vocabulary_create_no_name(self):
        '''Test error response when user tries to create a vocab without a
        name.

        '''
        params = {}
        param_string = helpers.json.dumps(params)
        response = self.app.post('/api/action/vocabulary_create',
                params=param_string,
                extra_environ={'Authorization':
                    str(self.sysadmin_user.apikey)},
                status=409)
        assert response.json['success'] is False
        assert response.json['error']['name'] == [u'Missing value']

    def test_vocabulary_create_invalid_name(self):
        '''Test error response when user tries to create a vocab with an
        invalid name.

        '''
        for name in (None, '', 'a', 'foobar' * 100):
            params = {'name': name}
            param_string = helpers.json.dumps(params)
            response = self.app.post('/api/action/vocabulary_create',
                    params=param_string,
                    extra_environ={'Authorization':
                        str(self.sysadmin_apikey)},
                    status=409)
            assert response.json['success'] is False
            assert response.json['error']['name']

    def test_vocabulary_create_exists(self):
        '''Test error response when user tries to create a vocab that already
        exists.

        '''
        params = {'name': self.genre_vocab['name']}
        param_string = helpers.json.dumps(params)
        response = self.app.post('/api/action/vocabulary_create',
                params=param_string,
                extra_environ={'Authorization':
                    str(self.sysadmin_user.apikey)},
                status=409)
        assert response.json['success'] is False
        assert response.json['error']['name'] == [u'That vocabulary name is '
            'already in use.']

    def test_vocabulary_create_not_logged_in(self):
        '''Test that users who are not logged in cannot create vocabularies.'''

        params = {'name':
            "Spam Vocabulary: SpamCo Duck Rental: Rent Your Ducks From Us!"}
        param_string = helpers.json.dumps(params)
        response = self.app.post('/api/action/vocabulary_create',
                params=param_string,
                status=403)
        assert response.json['success'] is False
        assert response.json['error']['__type'] == 'Authorization Error'

    def test_vocabulary_create_not_authorized(self):
        '''Test that users who are not authorized cannot create vocabs.'''

        params = {'name': 'My Unauthorised Vocabulary'}
        param_string = helpers.json.dumps(params)
        response = self.app.post('/api/action/vocabulary_create',
                params=param_string,
                extra_environ={'Authorization':
                    str(self.normal_user.apikey)},
                status=403)
        assert response.json['success'] is False
        assert response.json['error']['__type'] == 'Authorization Error'

    def test_vocabulary_update_id_only(self):
        self._update_vocabulary({'id': self.genre_vocab['id']},
                self.sysadmin_user)

    def test_vocabulary_update_id_and_same_name(self):
        self._update_vocabulary({'id': self.genre_vocab['id'],
            'name': self.genre_vocab['name']}, self.sysadmin_user)

    def test_vocabulary_update_id_and_new_name(self):
        self._update_vocabulary({'id': self.genre_vocab['id'],
            'name': 'new name'}, self.sysadmin_user)

    def test_vocabulary_update_id_and_same_tags(self):
        self._update_vocabulary({'id': self.genre_vocab['id'],
            'tags': self.genre_vocab['tags']}, self.sysadmin_user)

    def test_vocabulary_update_id_and_new_tags(self):
        tags = [
                {'name': 'new test tag one'},
                {'name': 'new test tag two'},
                {'name': 'new test tag three'},
                ]
        self._update_vocabulary({'id': self.genre_vocab['id'], 'tags': tags},
                self.sysadmin_user)

    def test_vocabulary_update_id_same_name_and_same_tags(self):
        self._update_vocabulary({'id': self.genre_vocab['id'],
            'name': self.genre_vocab['name'],
            'tags': self.genre_vocab['tags']}, self.sysadmin_user)

    def test_vocabulary_update_id_same_name_and_new_tags(self):
        tags = [
                {'name': 'new test tag one'},
                {'name': 'new test tag two'},
                {'name': 'new test tag three'},
                ]
        self._update_vocabulary({'id': self.genre_vocab['id'],
            'name': self.genre_vocab['name'],
            'tags': tags}, self.sysadmin_user)

    def test_vocabulary_update_id_new_name_and_same_tags(self):
        self._update_vocabulary({'id': self.genre_vocab['id'],
            'name': 'new name',
            'tags': self.genre_vocab['tags']}, self.sysadmin_user)

    def test_vocabulary_update_not_exists(self):
        '''Test the error response given when a user tries to update a
        vocabulary that doesn't exist.

        '''
        params = {'id': 'xxxxxxx', 'name': 'updated_name'}
        param_string = helpers.json.dumps(params)
        response = self.app.post('/api/action/vocabulary_update',
                params=param_string,
                extra_environ={'Authorization':
                    str(self.sysadmin_user.apikey)},
                status=404)
        assert response.json['success'] is False
        assert response.json['error']['message'].startswith('Not found: ')

    def test_vocabulary_update_no_id(self):
        params = {'name': 'bagel radio'}
        param_string = helpers.json.dumps(params)
        response = self.app.post('/api/action/vocabulary_update',
                params=param_string,
                extra_environ={'Authorization':
                    str(self.sysadmin_user.apikey)},
                status=409)
        assert response.json['success'] is False
        assert 'id' in response.json['error']
        assert response.json['error']['id'] == 'id not in data'

    def test_vocabulary_update_not_logged_in(self):
        '''Test that users who are not logged in cannot update vocabularies.'''
        params = {'id': self.genre_vocab['id']}
        param_string = helpers.json.dumps(params)
        response = self.app.post('/api/action/vocabulary_update',
                params=param_string,
                status=403)
        assert response.json['success'] is False
        assert response.json['error']['__type'] == 'Authorization Error'

    def test_vocabulary_update_with_tags(self):
        tags = [
                {'name': 'drone'},
                {'name': 'noise'},
                {'name': 'fuzz'},
                {'name': 'field recordings'},
                {'name': 'hypnagogia'},
                {'name': 'textures without rhythm'},
                ]
        self._update_vocabulary(
                {
                    'id': self.genre_vocab['id'],
                    'name': self.genre_vocab['name'],
                    'tags': tags
                },
                self.sysadmin_user)

        params = {'id': self.genre_vocab['id']}
        response = self._post('/api/action/vocabulary_show', params)
        # Check that retrieving the vocab by name gives the same result.
        assert len(response['result']['tags']) == len(tags)

    def test_vocabulary_update_not_authorized(self):
        '''Test that users who are not authorized cannot update vocabs.'''
        params = {'id': self.genre_vocab['id']}
        param_string = helpers.json.dumps(params)
        response = self.app.post('/api/action/vocabulary_update',
                params=param_string,
                extra_environ={'Authorization':
                    str(self.normal_user.apikey)},
                status=403)
        assert response.json['success'] is False
        assert response.json['error']['message'] == 'Access denied'

    def test_vocabulary_update_bad_tags(self):
        '''Test updating vocabularies with invalid tags.

        '''
        apikey = str(self.sysadmin_user.apikey)

        for tags in (
                [{'id': 'xxx'}, {'name': 'foo'}],
                [{'name': 'foo'}, {'name': None}],
                [{'name': 'foo'}, {'name': ''}],
                [{'name': 'foo'}, {'name': 'f'}],
                [{'name': 'f' * 200}, {'name': 'foo'}],
                [{'name': 'Invalid!'}, {'name': 'foo'}],
                ):
            params = {'id': self.genre_vocab['name'], 'tags': tags}
            response = self.app.post('/api/action/vocabulary_update',
                    params=helpers.json.dumps(params),
                    extra_environ={'Authorization': apikey},
                    status=409)
            assert response.json['success'] is False
            assert response.json['error']['tags']

    def test_vocabulary_update_none_tags(self):
        '''Test updating vocabularies with None for 'tags'.

        '''
        params = {'id': self.genre_vocab['id'], 'tags': None}
        response = self.app.post('/api/action/vocabulary_update',
                params=helpers.json.dumps(params),
                extra_environ={'Authorization':
                    str(self.sysadmin_user.apikey)},
                status=400)
        assert "Integrity Error" in response.body, response.body

    def test_vocabulary_update_empty_tags(self):
        '''Test updating vocabularies with [] for 'tags'.

        '''
        params = {'id': self.genre_vocab['id'], 'tags': []}
        response = self.app.post('/api/action/vocabulary_update',
                params=helpers.json.dumps(params),
                extra_environ={'Authorization':
                    str(self.sysadmin_user.apikey)},
                status=200)
        assert response.json['success'] is True
        assert response.json['result']
        updated_vocab = response.json['result']
        assert updated_vocab['name'] == self.genre_vocab['name']
        assert updated_vocab['id'] == self.genre_vocab['id']
        assert updated_vocab['tags'] == []
        params = {'id': updated_vocab['id']}
        response = self._post('/api/action/vocabulary_show', params)
        assert response['success'] is True
        assert response['result'] == updated_vocab
        tags = self._list_tags(updated_vocab)
        assert tags == []

    def test_vocabulary_delete(self):
        self._delete_vocabulary(self.genre_vocab['id'], self.sysadmin_user)

    def test_vocabulary_delete_not_exists(self):
        '''Test the error response given when a user tries to delete a
        vocabulary that doesn't exist.

        '''
        params = {'id': 'xxxxxxx'}
        param_string = helpers.json.dumps(params)
        response = self.app.post('/api/action/vocabulary_delete',
                params=param_string,
                extra_environ={'Authorization':
                    str(self.sysadmin_user.apikey)},
                status=404)
        assert response.json['success'] is False
        assert response.json['error']['message'].startswith('Not found: '
                'Could not find vocabulary')

    def test_vocabulary_delete_no_id(self):
        '''Test the error response given when a user tries to delete a
        vocabulary without giving the vocabulary id.

        '''
        params = {}
        param_string = helpers.json.dumps(params)
        response = self.app.post('/api/action/vocabulary_delete',
                params=param_string,
                extra_environ={'Authorization':
                    str(self.sysadmin_user.apikey)},
                status=409)
        assert response.json['success'] is False
        assert 'id' in response.json['error']
        assert response.json['error']['id'] == 'id not in data'

    def test_vocabulary_delete_not_logged_in(self):
        '''Test that users who are not logged in cannot delete vocabularies.'''
        params = {'id': self.genre_vocab['id']}
        param_string = helpers.json.dumps(params)
        response = self.app.post('/api/action/vocabulary_delete',
                params=param_string,
                status=403)
        assert response.json['success'] is False
        assert response.json['error']['__type'] == 'Authorization Error'

    def test_vocabulary_delete_not_authorized(self):
        '''Test that users who are not authorized cannot delete vocabs.'''
        params = {'id': self.genre_vocab['id']}
        param_string = helpers.json.dumps(params)
        response = self.app.post('/api/action/vocabulary_delete',
                params=param_string,
                extra_environ={'Authorization':
                    str(self.normal_user.apikey)},
                status=403)
        assert response.json['success'] is False
        assert response.json['error']['__type'] == 'Authorization Error'

    def test_add_tag_to_vocab(self):
        '''Test that a tag can be added to and then retrieved from a vocab.'''
        vocab = self.genre_vocab
        tags_before = self._list_tags(vocab)
        tag_created = self._create_tag(self.sysadmin_user, 'noise', vocab)
        tags_after = self._list_tags(vocab)
        new_tag_names = [tag_name for tag_name in tags_after if tag_name not in
                tags_before]
        assert len(new_tag_names) == 1
        assert tag_created['name'] in new_tag_names

    def test_add_tag_no_vocab(self):
        '''Test the error response when a user tries to create a tag without
        specifying a vocab.

        '''
        tag_dict = {'name': 'noise'}
        tag_string = helpers.json.dumps(tag_dict)
        response = self.app.post('/api/action/tag_create',
                params=tag_string,
                extra_environ={'Authorization':
                    str(self.sysadmin_user.apikey)},
                status=409)
        assert response.json['success'] is False
        assert response.json['error']['vocabulary_id'] == ['Missing value']

    def test_add_tag_vocab_not_exists(self):
        '''Test the error response when a user tries to add a tag to a vocab
        that doesn't exist.

        '''
        tag_dict = {'name': 'noise', 'vocabulary_id': 'does not exist'}
        tag_string = helpers.json.dumps(tag_dict)
        response = self.app.post('/api/action/tag_create',
                params=tag_string,
                extra_environ={'Authorization':
                    str(self.sysadmin_user.apikey)},
                status=409)
        assert response.json['success'] is False
        assert response.json['error']['vocabulary_id'] == [
                'Tag vocabulary was not found.']

    def test_add_tag_already_added(self):
        '''Test the error response when a user tries to add a tag to a vocab
        that already has a tag with the same name.

        '''
        self.test_add_tag_to_vocab()
        vocab = self.genre_vocab
        tag_dict = {'name': 'noise', 'vocabulary_id': vocab['id']}
        tag_string = helpers.json.dumps(tag_dict)
        response = self.app.post('/api/action/tag_create',
                params=tag_string,
                extra_environ={'Authorization':
                    str(self.sysadmin_user.apikey)},
                status=409)
        assert response.json['success'] is False
        assert response.json['error']['vocabulary_id'][0].startswith(
            'Tag noise already belongs to vocabulary')

    def test_add_tag_with_id(self):
        '''Test the error response when a user tries to specify the tag ID when
        adding a tag to a vocab.

        '''
        tag_dict = {
                'id': 'dsagdsgsgsd',
                'name': 'noise',
                'vocabulary_id': self.genre_vocab['id']
                }
        tag_string = helpers.json.dumps(tag_dict)
        response = self.app.post('/api/action/tag_create',
                params=tag_string,
                extra_environ={'Authorization':
                    str(self.sysadmin_user.apikey)},
                status=409)
        assert response.json['success'] is False
        assert response.json['error']['id'] == [u'The input field id was not '
            'expected.']

    def test_add_tag_without_name(self):
        '''Test the error response when a user tries to create a tag without a
        name.

        '''
        tag_dict = {
                'vocabulary_id': self.genre_vocab['id']
                }
        tag_string = helpers.json.dumps(tag_dict)
        response = self.app.post('/api/action/tag_create',
                params=tag_string,
                extra_environ={'Authorization':
                    str(self.sysadmin_user.apikey)},
                status=409)
        assert response.json['success'] is False
        assert response.json['error']['name'] == [u'Missing value']

    def test_add_tag_invalid_name(self):
        for name in ('Not a valid tag name!', '', None):
            tag_dict = {
                    'name': name,
                    'vocabulary_id': self.genre_vocab['id']
                    }
            tag_string = helpers.json.dumps(tag_dict)
            response = self.app.post('/api/action/tag_create',
                    params=tag_string,
                    extra_environ={'Authorization':
                        str(self.sysadmin_apikey)},
                    status=409)
            assert response.json['success'] is False
            assert response.json['error']['name']

    def test_add_tag_invalid_vocab_id(self):
        tag_dict = {
                'name': 'noise',
                'vocabulary_id': 'xxcxzczxczxc',
                }
        tag_string = helpers.json.dumps(tag_dict)
        response = self.app.post('/api/action/tag_create',
                params=tag_string,
                extra_environ={'Authorization':
                    str(self.sysadmin_user.apikey)},
                status=409)
        assert response.json['success'] is False
        assert response.json['error']['vocabulary_id'] == [
                u'Tag vocabulary was not found.']

    def test_add_tag_not_logged_in(self):
        tag_dict = {
                'name': 'noise',
                'vocabulary_id': self.genre_vocab['id']
                }
        tag_string = helpers.json.dumps(tag_dict)
        response = self.app.post('/api/action/tag_create',
                params=tag_string,
                status=403)
        assert response.json['success'] is False
        assert response.json['error']['__type'] == 'Authorization Error'

    def test_add_tag_not_authorized(self):
        tag_dict = {
                'name': 'noise',
                'vocabulary_id': self.genre_vocab['id']
                }
        tag_string = helpers.json.dumps(tag_dict)
        response = self.app.post('/api/action/tag_create',
                params=tag_string,
                    extra_environ={'Authorization':
                        str(self.normal_user.apikey)},
                status=403)
        assert response.json['success'] is False
        assert response.json['error']['__type'] == 'Authorization Error'

    def test_add_vocab_tag_to_dataset(self):
        '''Test that a tag belonging to a vocab can be added to a dataset,
        retrieved from the dataset, and then removed from the dataset.'''

        ckan.model.repo.rebuild_db()
        self.setup()
        ckan.tests.legacy.CreateTestData.create()
        # First add a tag to the vocab.
        vocab = self.genre_vocab
        tag = self._create_tag(self.sysadmin_user, 'noise', vocab)

        # Get a package from the API.
        package = (self._post('/api/action/package_show',
            {'id': self._post('/api/action/package_list')['result'][0]})
            ['result'])

        # Add the new vocab tag to the package.
        package['tags'].append(tag)

        updated_package = self._post('/api/action/package_update',
                params={'id': package['id'], 'tags': package['tags']},
                extra_environ={'Authorization':
                    str(self.sysadmin_user.apikey)})['result']

        # Test that the new vocab tag was added to the package.
        tags_in_pkg = [tag_in_pkg for tag_in_pkg in updated_package['tags'] if
                tag_in_pkg['name'] == tag['name'] and
                tag_in_pkg['vocabulary_id'] == tag['vocabulary_id']]
        assert len(tags_in_pkg) == 1

        # Test that the package appears in tag_show.
        noise_tag = self._post('/api/action/tag_show',
                               params={'id': 'noise',
                                       'vocabulary_id': vocab['id'],
                                       'include_datasets': True}
                               )['result']
        assert len([p for p in noise_tag['packages'] if
                    p['id'] == updated_package['id']]) == 1

        # Remove the new vocab tag from the package.
        package['tags'].remove(tag)
        updated_package = self._post('/api/action/package_update',
                params={'id': package['id'], 'tags': package['tags']},
                extra_environ={'Authorization':
                    str(self.sysadmin_user.apikey)})['result']

        # Test that the tag no longer appears in the list of tags for the
        # package.
        package = (self._post('/api/action/package_show',
            {'id': self._post('/api/action/package_list')['result'][0]})
            ['result'])
        tags_in_pkg = [tag_in_pkg for tag_in_pkg in package['tags'] if
                tag_in_pkg['name'] == tag['name'] and
                tag_in_pkg['vocabulary_id'] == tag['vocabulary_id']]
        assert len(tags_in_pkg) == 0

    def test_delete_tag_from_vocab(self):
        '''Test that a tag can be deleted from a vocab.'''

        ckan.model.repo.rebuild_db()
        self.setup()
        ckan.tests.legacy.CreateTestData.create()
        vocab = self.genre_vocab

        # First add some tags to the vocab.
        noise_tag = self._create_tag(self.sysadmin_user, 'noise', vocab)
        ragga_tag = self._create_tag(self.sysadmin_user, 'ragga', vocab)
        grunge_tag = self._create_tag(self.sysadmin_user, 'grunge', vocab)
        funk_tag = self._create_tag(self.sysadmin_user, 'funk', vocab)
        tags = (noise_tag, ragga_tag, grunge_tag, funk_tag)

        # Get a package from the API.
        package = (self._post('/api/action/package_show',
            {'id': self._post('/api/action/package_list')['result'][0]})
            ['result'])

        # Add the new vocab tags to the package.
        for tag in tags:
            package['tags'].append(tag)

        updated_package = self._post('/api/action/package_update',
                params={'id': package['id'], 'tags': package['tags']},
                extra_environ={'Authorization':
                    str(self.sysadmin_user.apikey)})['result']

        # Test that the new vocab tags were added to the package.
        for tag in tags:
            tags_in_pkg = [tag_in_pkg for tag_in_pkg in
                    updated_package['tags'] if tag_in_pkg['name'] ==
                    tag['name'] and tag_in_pkg['vocabulary_id'] ==
                    tag['vocabulary_id']]
            assert len(tags_in_pkg) == 1

        # Now delete the tags from the vocab.
        tags_before = self._list_tags(vocab)
        self._delete_tag(self.sysadmin_user, noise_tag['name'], vocab['name'])
        self._delete_tag(self.sysadmin_user, ragga_tag['id'], vocab['name'])
        self._delete_tag(self.sysadmin_user, grunge_tag['id'], vocab['id'])
        self._delete_tag(self.sysadmin_user, funk_tag['name'], vocab['id'])

        # Test that the tags no longer appear in the list of tags for the
        # vocab.
        tags_after = self._list_tags(vocab)
        assert len(tags_after) == len(tags_before) - 4
        assert tag['name'] not in tags_after
        difference = [tag_name for tag_name in tags_before if tag_name not in
                tags_after]
        assert sorted(difference) == sorted([tag['name'] for tag in tags])

        # Test that the tags no longer appear in the list of tags for the
        # package.
        package = (self._post('/api/action/package_show',
            {'id': self._post('/api/action/package_list')['result'][0]})
            ['result'])
        for tag in tags:
            tags_in_pkg = [tag_in_pkg for tag_in_pkg in package['tags'] if
                    tag_in_pkg['name'] == tag['name'] and
                    tag_in_pkg['vocabulary_id'] == tag['vocabulary_id']]
            assert len(tags_in_pkg) == 0

    def test_delete_free_tag(self):
        '''Test that a free tag can be deleted via the API, and is
        automatically removed from datasets.

        '''
        ckan.model.repo.rebuild_db()
        self.setup()
        ckan.tests.legacy.CreateTestData.create()
        # Get a package from the API.
        package = (self._post('/api/action/package_show',
            {'id': self._post('/api/action/package_list')['result'][0]})
            ['result'])
        package_id = package['id']

        # Add some new free tags to the package.
        tags = package['tags']
        tags.append({'name': 'ducks'})
        tags.append({'name': 'birds'})
        self._post('/api/action/package_update',
                params={'id': package['id'], 'tags': tags},
                extra_environ={'Authorization':
                    str(self.sysadmin_user.apikey)})

        # Test that the new tags appear in the list of tags.
        tags = self._list_tags()
        assert [tag for tag in tags].count('ducks') == 1
        assert [tag for tag in tags].count('birds') == 1

        # Test that the new tags appear in the package's list of tags.
        package = (self._post('/api/action/package_show',
            {'id': package_id})['result'])
        packages_tags = [tag['name'] for tag in package['tags']]
        assert [tag for tag in packages_tags].count('ducks') == 1
        assert [tag for tag in packages_tags].count('birds') == 1

        # Now delete the tags.
        self._delete_tag(self.sysadmin_user, 'ducks')
        birds_tag_id = self._post('/api/action/tag_show',
                {'id': 'birds'})['result']['id']
        self._delete_tag(self.sysadmin_user, birds_tag_id)

        # Test that the tags no longer appear in the list of tags.
        tags = self._list_tags()
        assert [tag for tag in tags].count('ducks') == 0
        assert [tag for tag in tags].count('birds') == 0

        # Test that the tags no longer appear in the package's list of tags.
        package = (self._post('/api/action/package_show',
            {'id': package_id})['result'])
        packages_tags = [tag['name'] for tag in package['tags']]
        assert [tag for tag in packages_tags].count('ducks') == 0
        assert [tag for tag in packages_tags].count('birds') == 0

    def test_delete_tag_no_id(self):
        '''Test the error response when a user tries to delete a tag without
        giving the tag id.

        '''
        vocab = self.genre_vocab
        self._create_tag(self.sysadmin_user, 'noise', vocab)

        for tag_id in ('missing', '', None):
            # Now try to delete the tag from the vocab.
            params = {'vocabulary_id': vocab['name']}
            if tag_id != 'missing':
                params['id'] = tag_id
            response = self.app.post('/api/action/tag_delete',
                    params=helpers.json.dumps(params),
                    extra_environ={'Authorization':
                        str(self.sysadmin_user.apikey)},
                    status=409)
            assert response.json['success'] is False
            assert 'id' in response.json['error']
            assert response.json['error']['id'] == 'id not in data'

    def test_delete_tag_no_vocab(self):
        '''Test the error response when a user tries to delete a vocab tag
        without giving the vocab name.

        '''
        vocab = self.genre_vocab
        tag = self._create_tag(self.sysadmin_user, 'noise', vocab)

        # Now try to delete the tag from the vocab.
        for vocab_name in ('', None, 'missing'):
            params = {'id': tag['name']}
            if vocab_name != 'missing':
                params['vocabulary_id'] = vocab_name
            response = self.app.post('/api/action/tag_delete',
                    params=helpers.json.dumps(params),
                    extra_environ={'Authorization':
                        str(self.sysadmin_user.apikey)},
                    status=404)
            assert response.json['success'] is False
            msg = response.json['error']['message']
            assert msg == u'Not found: Could not find tag "{0}"'.format(
                    tag['name']), msg

    def test_delete_tag_not_exists(self):
        '''Test the error response when a user tries to delete a from a vocab
        but there is no tag with that name in the vocab.

        '''
        vocab = self.genre_vocab
        self._create_tag(self.sysadmin_user, 'noise', vocab)

        params = {'id': 'nonexistent',
                'vocabulary_id': self.genre_vocab['name']}
        response = self.app.post('/api/action/tag_delete',
                params=helpers.json.dumps(params),
                extra_environ={'Authorization':
                    str(self.sysadmin_user.apikey)},
                status=404)
        assert response.json['success'] is False
        msg = response.json['error']['message']
        assert msg == u'Not found: Could not find tag "%s"' % 'nonexistent', \
                msg

    def test_delete_tag_vocab_not_exists(self):
        '''Test the error response when a user tries to delete a from a vocab
        but there is no vocab with that name.

        '''
        vocab = self.genre_vocab
        tag = self._create_tag(self.sysadmin_user, 'noise', vocab)

        params = {'id': tag['name'],
                'vocabulary_id': 'nonexistent'}
        response = self.app.post('/api/action/tag_delete',
                params=helpers.json.dumps(params),
                extra_environ={'Authorization':
                    str(self.sysadmin_user.apikey)},
                status=404)
        assert response.json['success'] is False
        msg = response.json['error']['message']
        assert msg == u"Not found: could not find vocabulary 'nonexistent'", \
                msg

    def test_delete_tag_invalid_tag(self):
        '''Test the error response when a user tries to delete a tag but gives
        an invalid tag name.

        '''
        vocab = self.genre_vocab
        self._create_tag(self.sysadmin_user, 'noise', vocab)

        for tag_name in ('Invalid!', ' '):
            params = {'id': tag_name,
                    'vocabulary_id': self.genre_vocab['name']}
            response = self.app.post('/api/action/tag_delete',
                    params=helpers.json.dumps(params),
                    extra_environ={'Authorization':
                        str(self.sysadmin_user.apikey)},
                    status=404)
            assert response.json['success'] is False
            msg = response.json['error']['message']
            assert msg == u'Not found: Could not find tag "%s"' % tag_name, msg

    def test_delete_tag_invalid_vocab(self):
        '''Test the error response when a user tries to delete a tag but gives
        an invalid vocab name.

        '''
        vocab = self.genre_vocab
        tag = self._create_tag(self.sysadmin_user, 'noise', vocab)

        for vocab_name in ('Invalid!', ' '):
            params = {'id': tag['name'], 'vocabulary_id': vocab_name}
            response = self.app.post('/api/action/tag_delete',
                    params=helpers.json.dumps(params),
                    extra_environ={'Authorization':
                        str(self.sysadmin_user.apikey)},
                    status=404)
            assert response.json['success'] is False
            msg = response.json['error']['message']
            assert msg == u"Not found: could not find vocabulary '%s'" \
                    % vocab_name, msg

    def test_delete_tag_not_logged_in(self):
        vocab = self.genre_vocab
        tag = self._create_tag(self.sysadmin_user, 'noise', vocab)

        params = {'id': tag['name'],
                'vocabulary_id': self.genre_vocab['name']}
        response = self.app.post('/api/action/tag_delete',
                params=helpers.json.dumps(params),
                status=403)
        assert response.json['success'] is False
        error = response.json['error']['__type']
        assert error == u"Authorization Error", error

    def test_delete_tag_not_authorized(self):
        vocab = self.genre_vocab
        tag = self._create_tag(self.sysadmin_user, 'noise', vocab)

        params = {'id': tag['name'],
                'vocabulary_id': self.genre_vocab['name']}
        response = self.app.post('/api/action/tag_delete',
                params=helpers.json.dumps(params),
                extra_environ={'Authorization':
                    str(self.normal_user.apikey)},
                status=403)
        assert response.json['success'] is False
        msg = response.json['error']['__type']
        assert msg == u"Authorization Error"



# encoding: utf-8

from sqlalchemy import *
from migrate import *

def upgrade(migrate_engine):
    metadata = MetaData()
    metadata.bind = migrate_engine
    user_table = Table('user', metadata, autoload=True)
    reset_key_col = Column('reset_key', UnicodeText)
    reset_key_col.create(user_table)

def downgrade(migrate_engine):
    raise NotImplementedError()




# encoding: utf-8

import ckan.lib.helpers as h

class TestFormatText:

    def test_markdown(self):
        instr = '''# Hello World

**Some bolded text.**

*Some italicized text.*
'''
        exp = '''<h1>Hello World</h1>
<p><strong>Some bolded text.</strong></p>
<p><em>Some italicized text.</em></p>'''
        out = h.render_markdown(instr)
        assert out == exp, out

    def test_markdown_blank(self):
        instr = None
        out = h.render_markdown(instr)
        assert out == ''

    def test_evil_markdown(self):
        instr = 'Evil <script src="http://evilserver.net/evil.js";>'
        exp = '''<p>Evil </p>'''
        out = h.render_markdown(instr)
        assert out == exp, out

    def test_internal_link(self):
        instr = 'dataset:test-_pkg'
        exp = '<p><a href="/dataset/test-_pkg">dataset:test-_pkg</a></p>'
        out = h.render_markdown(instr)
        assert exp in out, '\nGot: %s\nWanted: %s' % (out, exp)

    def test_internal_tag_link(self):
        """Asserts links like 'tag:test-tag' work"""
        instr = 'tag:test-tag foobar'
        exp = '<a href="/tag/test-tag">tag:test-tag</a> foobar'
        out = h.render_markdown(instr)
        assert exp in out, '\nGot: %s\nWanted: %s' % (out, exp)

    def test_internal_tag_linked_with_quotes(self):
        """Asserts links like 'tag:"test-tag"' work"""
        instr = 'tag:"test-tag" foobar'
        exp = '<p><a href="/tag/test-tag">tag:&#34;test-tag&#34;</a> foobar</p>'
        out = h.render_markdown(instr)
        assert exp in out, '\nGot: %s\nWanted: %s' % (out, exp)

    def test_internal_tag_linked_with_quotes_and_space(self):
        """Asserts links like 'tag:"test tag"' work"""
        instr = 'tag:"test tag" foobar'
        exp = '<p><a href="/tag/test%20tag">tag:&#34;test tag&#34;</a> foobar</p>'
        out = h.render_markdown(instr)
        assert exp in out, '\nGot: %s\nWanted: %s' % (out, exp)

    def test_internal_tag_with_no_opening_quote_only_matches_single_word(self):
        """Asserts that without an opening quote only one word is matched"""
        instr = 'tag:test tag" foobar'  # should match 'tag:test'
        exp = '<a href="/tag/test">tag:test</a> tag" foobar'
        out = h.render_markdown(instr)
        assert exp in out, '\nGot: %s\nWanted: %s' % (out, exp)

    def test_internal_tag_with_no_opening_quote_wont_match_the_closing_quote(self):
        """Asserts that 'tag:test" tag' is matched, but to 'tag:test'"""
        instr = 'tag:test" foobar'  # should match 'tag:test'
        exp = '<a href="/tag/test">tag:test</a>" foobar'
        out = h.render_markdown(instr)
        assert exp in out, '\nGot: %s\nWanted: %s' % (out, exp)

    def test_internal_tag_with_no_closing_quote_does_not_match(self):
        """Asserts that without an opening quote only one word is matched"""
        instr = 'tag:"test tag foobar'
        out = h.render_markdown(instr)
        assert "<a href" not in out

    def test_tag_names_match_simple_punctuation(self):
        """Asserts punctuation and capital letters are matched in the tag name"""
        instr = 'tag:"Test- _." foobar'
        exp = '<p><a href="/tag/Test-%20_.">tag:&#34;Test- _.&#34;</a> foobar</p>'
        out = h.render_markdown(instr)
        assert exp in out, '\nGot: %s\nWanted: %s' % (out, exp)

    def test_tag_names_do_not_match_commas(self):
        """Asserts commas don't get matched as part of a tag name"""
        instr = 'tag:Test,tag foobar'
        exp = '<a href="/tag/Test">tag:Test</a>,tag foobar'
        out = h.render_markdown(instr)
        assert exp in out, '\nGot: %s\nWanted: %s' % (out, exp)

    def test_tag_names_dont_match_non_space_whitespace(self):
        """Asserts that the only piece of whitespace matched in a tagname is a space"""
        whitespace_characters = '\t\n\r\f\v'
        for ch in whitespace_characters:
            instr = 'tag:Bad' + ch + 'space'
            exp = '<a href="/tag/Bad">tag:Bad</a>'
            out = h.render_markdown(instr)
            assert exp in out, '\nGot: %s\nWanted: %s' % (out, exp)

    def test_tag_names_with_unicode_alphanumeric(self):
        """Asserts that unicode alphanumeric characters are captured"""
        instr = u'tag:"Japanese katakana \u30a1" blah'
        exp = u'<p><a href="/tag/Japanese%20katakana%20%E3%82%A1">tag:&#34;Japanese katakana \u30a1&#34;</a> blah</p>'
        out = h.render_markdown(instr)
        assert exp in out, u'\nGot: %s\nWanted: %s' % (out, exp)

    def test_normal_link(self):
        instr = 'http://somelink/'
        exp = '<a href="http://somelink/" target="_blank" rel="nofollow">http://somelink/</a>'
        out = h.render_markdown(instr)
        assert exp in out, '\nGot: %s\nWanted: %s' % (out, exp)

        instr = 'http://somelink.com/#anchor'
        exp = '<a href="http://somelink.com/#anchor" target="_blank" rel="nofollow">http://somelink.com/#anchor</a>'
        out = h.render_markdown(instr)
        assert exp in out, '\nGot: %s\nWanted: %s' % (out, exp)

        instr = 'http://www.somelink.com/#anchor'
        exp = '<a href="http://www.somelink.com/#anchor" target="_blank" rel="nofollow">http://www.somelink.com/#anchor</a>'
        out = h.render_markdown(instr)
        assert exp in out, '\nGot: %s\nWanted: %s' % (out, exp)

    def test_auto_link(self):
        instr = 'http://somelink.com'
        exp = '<a href="http://somelink.com" target="_blank" rel="nofollow">http://somelink.com</a>'
        out = h.render_markdown(instr)
        assert exp in out, '\nGot: %s\nWanted: %s' % (out, exp)

    def test_auto_link_after_whitespace(self):
        instr = 'go to http://somelink.com'
        exp = 'go to <a href="http://somelink.com" target="_blank" rel="nofollow">http://somelink.com</a>'
        out = h.render_markdown(instr)
        assert exp in out, '\nGot: %s\nWanted: %s' % (out, exp)

    def test_malformed_link_1(self):
        instr = u'<a href=\u201dsomelink\u201d>somelink</a>'
        exp = '<p>somelink</p>'
        out = h.render_markdown(instr)
        assert exp in out, '\nGot: %s\nWanted: %s' % (out, exp)

    def test_multiline_links(self):
        instr = u'''I get 10 times more traffic from [Google][] than from
[Yahoo][] or [MSN][].

  [google]: http://google.com/        "Google"
  [yahoo]:  http://search.yahoo.com/  "Yahoo Search"
  [msn]:    http://search.msn.com/    "MSN Search"'''
        exp = '''<p>I get 10 times more traffic from <a href="http://google.com/" title="Google">Google</a> than from
<a href="http://search.yahoo.com/" title="Yahoo Search">Yahoo</a> or <a href="http://search.msn.com/" title="MSN Search">MSN</a>.</p>'''
        out = h.render_markdown(instr)
        assert exp in out, '\nGot: %s\nWanted: %s' % (out, exp)



# encoding: utf-8

from bs4 import BeautifulSoup
from nose.tools import (
    assert_equal,
    assert_not_equal,
    assert_raises,
    assert_true,
    assert_in
)

from mock import patch, MagicMock
from routes import url_for

import ckan.model as model
import ckan.plugins as p
from ckan.lib import search

import ckan.tests.helpers as helpers
import ckan.tests.factories as factories


webtest_submit = helpers.webtest_submit
submit_and_follow = helpers.submit_and_follow


def _get_package_new_page(app):
    user = factories.User()
    env = {'REMOTE_USER': user['name'].encode('ascii')}
    response = app.get(
        url=url_for(controller='package', action='new'),
        extra_environ=env,
    )
    return env, response


class TestPackageNew(helpers.FunctionalTestBase):
    def test_form_renders(self):
        app = self._get_test_app()
        env, response = _get_package_new_page(app)
        assert_true('dataset-edit' in response.forms)

    @helpers.change_config('ckan.auth.create_unowned_dataset', 'false')
    def test_needs_organization_but_no_organizations_has_button(self):
        ''' Scenario: The settings say every dataset needs an organization
        but there are no organizations. If the user is allowed to create an
        organization they should be prompted to do so when they try to create
        a new dataset'''
        app = self._get_test_app()
        sysadmin = factories.Sysadmin()

        env = {'REMOTE_USER': sysadmin['name'].encode('ascii')}
        response = app.get(
            url=url_for(controller='package', action='new'),
            extra_environ=env
        )
        assert 'dataset-edit' not in response.forms
        assert url_for(controller='organization', action='new') in response

    @helpers.mock_auth('ckan.logic.auth.create.package_create')
    @helpers.change_config('ckan.auth.create_unowned_dataset', 'false')
    @helpers.change_config('ckan.auth.user_create_organizations', 'false')
    def test_needs_organization_but_no_organizations_no_button(self,
                                                               mock_p_create):
        ''' Scenario: The settings say every dataset needs an organization
        but there are no organizations. If the user is not allowed to create an
        organization they should be told to ask the admin but no link should be
        presented. Note: This cannot happen with the default ckan and requires
        a plugin to overwrite the package_create behavior'''
        mock_p_create.return_value = {'success': True}

        app = self._get_test_app()
        user = factories.User()

        env = {'REMOTE_USER': user['name'].encode('ascii')}
        response = app.get(
            url=url_for(controller='package', action='new'),
            extra_environ=env
        )

        assert 'dataset-edit' not in response.forms
        assert url_for(controller='organization', action='new') not in response
        assert 'Ask a system administrator' in response

    def test_name_required(self):
        app = self._get_test_app()
        env, response = _get_package_new_page(app)
        form = response.forms['dataset-edit']

        response = webtest_submit(form, 'save', status=200, extra_environ=env)
        assert_true('dataset-edit' in response.forms)
        assert_true('Name: Missing value' in response)

    def test_resource_form_renders(self):
        app = self._get_test_app()
        env, response = _get_package_new_page(app)
        form = response.forms['dataset-edit']
        form['name'] = u'resource-form-renders'

        response = submit_and_follow(app, form, env, 'save')
        assert_true('resource-edit' in response.forms)

    def test_first_page_creates_draft_package(self):
        app = self._get_test_app()
        env, response = _get_package_new_page(app)
        form = response.forms['dataset-edit']
        form['name'] = u'first-page-creates-draft'

        webtest_submit(form, 'save', status=302, extra_environ=env)
        pkg = model.Package.by_name(u'first-page-creates-draft')
        assert_equal(pkg.state, 'draft')

    def test_resource_required(self):
        app = self._get_test_app()
        env, response = _get_package_new_page(app)
        form = response.forms['dataset-edit']
        form['name'] = u'one-resource-required'

        response = submit_and_follow(app, form, env, 'save')
        form = response.forms['resource-edit']

        response = webtest_submit(form, 'save', value='go-metadata',
                                  status=200, extra_environ=env)
        assert_true('resource-edit' in response.forms)
        assert_true('You must add at least one data resource' in response)

    def test_complete_package_with_one_resource(self):
        app = self._get_test_app()
        env, response = _get_package_new_page(app)
        form = response.forms['dataset-edit']
        form['name'] = u'complete-package-with-one-resource'

        response = submit_and_follow(app, form, env, 'save')
        form = response.forms['resource-edit']
        form['url'] = u'http://example.com/resource'

        submit_and_follow(app, form, env, 'save', 'go-metadata')
        pkg = model.Package.by_name(u'complete-package-with-one-resource')
        assert_equal(pkg.resources[0].url, u'http://example.com/resource')
        assert_equal(pkg.state, 'active')

    def test_complete_package_with_two_resources(self):
        app = self._get_test_app()
        env, response = _get_package_new_page(app)
        form = response.forms['dataset-edit']
        form['name'] = u'complete-package-with-two-resources'

        response = submit_and_follow(app, form, env, 'save')
        form = response.forms['resource-edit']
        form['url'] = u'http://example.com/resource0'

        response = submit_and_follow(app, form, env, 'save', 'again')
        form = response.forms['resource-edit']
        form['url'] = u'http://example.com/resource1'

        submit_and_follow(app, form, env, 'save', 'go-metadata')
        pkg = model.Package.by_name(u'complete-package-with-two-resources')
        assert_equal(pkg.resources[0].url, u'http://example.com/resource0')
        assert_equal(pkg.resources[1].url, u'http://example.com/resource1')
        assert_equal(pkg.state, 'active')

    # def test_resource_uploads(self):
    #     app = self._get_test_app()
    #     env, response = _get_package_new_page(app)
    #     form = response.forms['dataset-edit']
    #     form['name'] = u'complete-package-with-two-resources'

    #     response = submit_and_follow(app, form, env, 'save')
    #     form = response.forms['resource-edit']
    #     form['upload'] = ('README.rst', b'data')

    #     response = submit_and_follow(app, form, env, 'save', 'go-metadata')
    #     pkg = model.Package.by_name(u'complete-package-with-two-resources')
    #     assert_equal(pkg.resources[0].url_type, u'upload')
    #     assert_equal(pkg.state, 'active')
    #     response = app.get(
    #         url_for(
    #             controller='package',
    #             action='resource_download',
    #             id=pkg.id,
    #             resource_id=pkg.resources[0].id
    #         ),
    #     )
    #     assert_equal('data', response.body)

    def test_previous_button_works(self):
        app = self._get_test_app()
        env, response = _get_package_new_page(app)
        form = response.forms['dataset-edit']
        form['name'] = u'previous-button-works'

        response = submit_and_follow(app, form, env, 'save')
        form = response.forms['resource-edit']

        response = submit_and_follow(app, form, env, 'save', 'go-dataset')
        assert_true('dataset-edit' in response.forms)

    def test_previous_button_populates_form(self):
        app = self._get_test_app()
        env, response = _get_package_new_page(app)
        form = response.forms['dataset-edit']
        form['name'] = u'previous-button-populates-form'

        response = submit_and_follow(app, form, env, 'save')
        form = response.forms['resource-edit']

        response = submit_and_follow(app, form, env, 'save', 'go-dataset')
        form = response.forms['dataset-edit']
        assert_true('title' in form.fields)
        assert_equal(form['name'].value, u'previous-button-populates-form')

    def test_previous_next_maintains_draft_state(self):
        app = self._get_test_app()
        env, response = _get_package_new_page(app)
        form = response.forms['dataset-edit']
        form['name'] = u'previous-next-maintains-draft'

        response = submit_and_follow(app, form, env, 'save')
        form = response.forms['resource-edit']

        response = submit_and_follow(app, form, env, 'save', 'go-dataset')
        form = response.forms['dataset-edit']

        webtest_submit(form, 'save', status=302, extra_environ=env)
        pkg = model.Package.by_name(u'previous-next-maintains-draft')
        assert_equal(pkg.state, 'draft')

    def test_dataset_edit_org_dropdown_visible_to_normal_user_with_orgs_available(self):
        '''
        The 'Organization' dropdown is available on the dataset create/edit
        page to normal (non-sysadmin) users who have organizations available
        to them.
        '''
        user = factories.User()
        # user is admin of org.
        org = factories.Organization(
            name="my-org",
            users=[{'name': user['id'], 'capacity': 'admin'}]
        )

        app = self._get_test_app()
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        response = app.get(
            url=url_for(controller='package', action='new'),
            extra_environ=env,
        )

        # organization dropdown available in create page.
        form = response.forms['dataset-edit']
        assert 'owner_org' in form.fields

        # create dataset
        form['name'] = u'my-dataset'
        form['owner_org'] = org['id']
        response = submit_and_follow(app, form, env, 'save')

        # add a resource to make the pkg active
        resource_form = response.forms['resource-edit']
        resource_form['url'] = u'http://example.com/resource'
        submit_and_follow(app, resource_form, env, 'save', 'go-metadata')
        pkg = model.Package.by_name(u'my-dataset')
        assert_equal(pkg.state, 'active')

        # edit package page response
        url = url_for(controller='package',
                      action='edit',
                      id=pkg.id)
        pkg_edit_response = app.get(url=url, extra_environ=env)
        # A field with the correct id is in the response
        form = pkg_edit_response.forms['dataset-edit']
        assert 'owner_org' in form.fields
        # The organization id is in the response in a value attribute
        owner_org_options = [value for (value, _) in form['owner_org'].options]
        assert org['id'] in owner_org_options

    def test_dataset_edit_org_dropdown_normal_user_can_remove_org(self):
        '''
        A normal user (non-sysadmin) can remove an organization from a dataset
        have permissions on.
        '''
        user = factories.User()
        # user is admin of org.
        org = factories.Organization(name="my-org",
                                     users=[{'name': user['id'], 'capacity': 'admin'}])

        app = self._get_test_app()
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        response = app.get(
            url=url_for(controller='package', action='new'),
            extra_environ=env,
        )

        # create dataset with owner_org
        form = response.forms['dataset-edit']
        form['name'] = u'my-dataset'
        form['owner_org'] = org['id']
        response = submit_and_follow(app, form, env, 'save')

        # add a resource to make the pkg active
        resource_form = response.forms['resource-edit']
        resource_form['url'] = u'http://example.com/resource'
        submit_and_follow(app, resource_form, env, 'save', 'go-metadata')
        pkg = model.Package.by_name(u'my-dataset')
        assert_equal(pkg.state, 'active')
        assert_equal(pkg.owner_org, org['id'])
        assert_not_equal(pkg.owner_org, None)

        # edit package page response
        url = url_for(controller='package',
                      action='edit',
                      id=pkg.id)
        pkg_edit_response = app.get(url=url, extra_environ=env)

        # edit dataset
        edit_form = pkg_edit_response.forms['dataset-edit']
        edit_form['owner_org'] = ''
        submit_and_follow(app, edit_form, env, 'save')
        post_edit_pkg = model.Package.by_name(u'my-dataset')
        assert_equal(post_edit_pkg.owner_org, None)
        assert_not_equal(post_edit_pkg.owner_org, org['id'])

    def test_dataset_edit_org_dropdown_not_visible_to_normal_user_with_no_orgs_available(self):
        '''
        The 'Organization' dropdown is not available on the dataset
        create/edit page to normal (non-sysadmin) users who have no
        organizations available to them.
        '''
        user = factories.User()
        # user isn't admin of org.
        org = factories.Organization(name="my-org")

        app = self._get_test_app()
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        response = app.get(
            url=url_for(controller='package', action='new'),
            extra_environ=env,
        )

        # organization dropdown not in create page.
        form = response.forms['dataset-edit']
        assert 'owner_org' not in form.fields

        # create dataset
        form['name'] = u'my-dataset'
        response = submit_and_follow(app, form, env, 'save')

        # add a resource to make the pkg active
        resource_form = response.forms['resource-edit']
        resource_form['url'] = u'http://example.com/resource'
        submit_and_follow(app, resource_form, env, 'save', 'go-metadata')
        pkg = model.Package.by_name(u'my-dataset')
        assert_equal(pkg.state, 'active')

        # edit package response
        url = url_for(controller='package',
                      action='edit',
                      id=model.Package.by_name(u'my-dataset').id)
        pkg_edit_response = app.get(url=url, extra_environ=env)
        # A field with the correct id is in the response
        form = pkg_edit_response.forms['dataset-edit']
        assert 'owner_org' not in form.fields
        # The organization id is in the response in a value attribute
        assert 'value="{0}"'.format(org['id']) not in pkg_edit_response

    def test_dataset_edit_org_dropdown_visible_to_sysadmin_with_no_orgs_available(self):
        '''
        The 'Organization' dropdown is available to sysadmin users regardless
        of whether they personally have an organization they administrate.
        '''
        user = factories.User()
        sysadmin = factories.Sysadmin()
        # user is admin of org.
        org = factories.Organization(name="my-org",
                                     users=[{'name': user['id'], 'capacity': 'admin'}])

        app = self._get_test_app()
        # user in env is sysadmin
        env = {'REMOTE_USER': sysadmin['name'].encode('ascii')}
        response = app.get(
            url=url_for(controller='package', action='new'),
            extra_environ=env,
        )

        # organization dropdown available in create page.
        assert 'id="field-organizations"' in response

        # create dataset
        form = response.forms['dataset-edit']
        form['name'] = u'my-dataset'
        form['owner_org'] = org['id']
        response = submit_and_follow(app, form, env, 'save')

        # add a resource to make the pkg active
        resource_form = response.forms['resource-edit']
        resource_form['url'] = u'http://example.com/resource'
        submit_and_follow(app, resource_form, env, 'save', 'go-metadata')
        pkg = model.Package.by_name(u'my-dataset')
        assert_equal(pkg.state, 'active')

        # edit package page response
        url = url_for(controller='package',
                      action='edit',
                      id=pkg.id)
        pkg_edit_response = app.get(url=url, extra_environ=env)
        # A field with the correct id is in the response
        assert 'id="field-organizations"' in pkg_edit_response
        # The organization id is in the response in a value attribute
        assert 'value="{0}"'.format(org['id']) in pkg_edit_response

    def test_unauthed_user_creating_dataset(self):
        app = self._get_test_app()

        # provide REMOTE_ADDR to idenfity as remote user, see
        # BaseController._identify_user() for details
        response = app.post(url=url_for(controller='package', action='new'),
                            extra_environ={'REMOTE_ADDR': '127.0.0.1'},
                            status=403)


class TestPackageEdit(helpers.FunctionalTestBase):
    def test_organization_admin_can_edit(self):
        user = factories.User()
        organization = factories.Organization(
            users=[{'name': user['id'], 'capacity': 'admin'}]
        )
        dataset = factories.Dataset(owner_org=organization['id'])
        app = helpers._get_test_app()
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        response = app.get(
            url_for(controller='package',
                    action='edit',
                    id=dataset['name']),
            extra_environ=env,
        )
        form = response.forms['dataset-edit']
        form['notes'] = u'edited description'
        submit_and_follow(app, form, env, 'save')

        result = helpers.call_action('package_show', id=dataset['id'])
        assert_equal(u'edited description', result['notes'])

    def test_organization_editor_can_edit(self):
        user = factories.User()
        organization = factories.Organization(
            users=[{'name': user['id'], 'capacity': 'editor'}]
        )
        dataset = factories.Dataset(owner_org=organization['id'])
        app = helpers._get_test_app()
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        response = app.get(
            url_for(controller='package',
                    action='edit',
                    id=dataset['name']),
            extra_environ=env,
        )
        form = response.forms['dataset-edit']
        form['notes'] = u'edited description'
        submit_and_follow(app, form, env, 'save')

        result = helpers.call_action('package_show', id=dataset['id'])
        assert_equal(u'edited description', result['notes'])

    def test_organization_member_cannot_edit(self):
        user = factories.User()
        organization = factories.Organization(
            users=[{'name': user['id'], 'capacity': 'member'}]
        )
        dataset = factories.Dataset(owner_org=organization['id'])
        app = helpers._get_test_app()
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        response = app.get(
            url_for(controller='package',
                    action='edit',
                    id=dataset['name']),
            extra_environ=env,
            status=403,
        )

    def test_user_not_in_organization_cannot_edit(self):
        user = factories.User()
        organization = factories.Organization()
        dataset = factories.Dataset(owner_org=organization['id'])
        app = helpers._get_test_app()
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        response = app.get(
            url_for(controller='package',
                    action='edit',
                    id=dataset['name']),
            extra_environ=env,
            status=403,
        )

        env = {'REMOTE_USER': user['name'].encode('ascii')}
        response = app.post(
            url_for(controller='package',
                    action='edit',
                    id=dataset['name']),
            {'notes': 'edited description'},
            extra_environ=env,
            status=403,
        )

    def test_anonymous_user_cannot_edit(self):
        organization = factories.Organization()
        dataset = factories.Dataset(owner_org=organization['id'])
        app = helpers._get_test_app()
        response = app.get(
            url_for(controller='package',
                    action='edit',
                    id=dataset['name']),
            status=403,
        )

        response = app.post(
            url_for(controller='package',
                    action='edit',
                    id=dataset['name']),
            {'notes': 'edited description'},
            status=403,
        )

    def test_validation_errors_for_dataset_name_appear(self):
        '''fill out a bad dataset set name and make sure errors appear'''
        user = factories.User()
        organization = factories.Organization(
            users=[{'name': user['id'], 'capacity': 'admin'}]
        )
        dataset = factories.Dataset(owner_org=organization['id'])
        app = helpers._get_test_app()
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        response = app.get(
            url_for(controller='package',
                    action='edit',
                    id=dataset['name']),
            extra_environ=env,
        )
        form = response.forms['dataset-edit']
        form['name'] = u'this is not a valid name'
        response = webtest_submit(form, 'save', status=200, extra_environ=env)
        assert_in('The form contains invalid entries', response.body)

        assert_in('Name: Must be purely lowercase alphanumeric (ascii) '
                  'characters and these symbols: -_', response.body)

    def test_edit_a_dataset_that_does_not_exist_404s(self):
        user = factories.User()
        app = helpers._get_test_app()
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        response = app.get(
            url_for(controller='package',
                    action='edit',
                    id='does-not-exist'),
            extra_environ=env,
            expect_errors=True
        )
        assert_equal(404, response.status_int)


class TestPackageRead(helpers.FunctionalTestBase):
    def test_read(self):
        dataset = factories.Dataset()
        app = helpers._get_test_app()
        response = app.get(url_for(controller='package', action='read',
                                   id=dataset['name']))
        response.mustcontain('Test Dataset')
        response.mustcontain('Just another test dataset')

    def test_organization_members_can_read_private_datasets(self):
        members = {
            'member': factories.User(),
            'editor': factories.User(),
            'admin': factories.User(),
            'sysadmin': factories.Sysadmin()
        }
        organization = factories.Organization(
            users=[
                {'name': members['member']['id'], 'capacity': 'member'},
                {'name': members['editor']['id'], 'capacity': 'editor'},
                {'name': members['admin']['id'], 'capacity': 'admin'},
            ]
        )
        dataset = factories.Dataset(
            owner_org=organization['id'],
            private=True,
        )
        app = helpers._get_test_app()

        for user, user_dict in members.items():
            response = app.get(
                url_for(
                    controller='package',
                    action='read',
                    id=dataset['name']
                ),
                extra_environ={
                    'REMOTE_USER': user_dict['name'].encode('ascii'),
                },
            )
            assert_in('Test Dataset', response.body)
            assert_in('Just another test dataset', response.body)

    def test_anonymous_users_cannot_read_private_datasets(self):
        organization = factories.Organization()
        dataset = factories.Dataset(
            owner_org=organization['id'],
            private=True,
        )
        app = helpers._get_test_app()
        response = app.get(
            url_for(controller='package', action='read', id=dataset['name']),
            status=404
        )
        assert_equal(404, response.status_int)

    def test_user_not_in_organization_cannot_read_private_datasets(self):
        user = factories.User()
        organization = factories.Organization()
        dataset = factories.Dataset(
            owner_org=organization['id'],
            private=True,
        )
        app = helpers._get_test_app()
        response = app.get(
            url_for(controller='package', action='read', id=dataset['name']),
            extra_environ={'REMOTE_USER': user['name'].encode('ascii')},
            status=404
        )
        assert_equal(404, response.status_int)

    def test_read_rdf(self):
        ''' The RDF outputs now live in ckanext-dcat'''
        dataset1 = factories.Dataset()

        offset = url_for(controller='package', action='read',
                         id=dataset1['name']) + ".rdf"
        app = self._get_test_app()
        app.get(offset, status=404)

    def test_read_n3(self):
        ''' The RDF outputs now live in ckanext-dcat'''
        dataset1 = factories.Dataset()

        offset = url_for(controller='package', action='read',
                         id=dataset1['name']) + ".n3"
        app = self._get_test_app()
        app.get(offset, status=404)


class TestPackageDelete(helpers.FunctionalTestBase):
    def test_owner_delete(self):
        user = factories.User()
        owner_org = factories.Organization(
            users=[{'name': user['id'], 'capacity': 'admin'}]
        )
        dataset = factories.Dataset(owner_org=owner_org['id'])

        app = helpers._get_test_app()
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        response = app.post(
            url_for(controller='package', action='delete', id=dataset['name']),
            extra_environ=env,
        )
        response = response.follow()
        assert_equal(200, response.status_int)

        deleted = helpers.call_action('package_show', id=dataset['id'])
        assert_equal('deleted', deleted['state'])

    def test_delete_on_non_existing_dataset(self):
        app = helpers._get_test_app()
        response = app.post(
            url_for(controller='package', action='delete',
                    id='schrodingersdatset'),
            expect_errors=True,
        )
        assert_equal(404, response.status_int)

    def test_sysadmin_can_delete_any_dataset(self):
        owner_org = factories.Organization()
        dataset = factories.Dataset(owner_org=owner_org['id'])
        app = helpers._get_test_app()

        user = factories.Sysadmin()
        env = {'REMOTE_USER': user['name'].encode('ascii')}

        response = app.post(
            url_for(controller='package', action='delete', id=dataset['name']),
            extra_environ=env,
        )
        response = response.follow()
        assert_equal(200, response.status_int)

        deleted = helpers.call_action('package_show', id=dataset['id'])
        assert_equal('deleted', deleted['state'])

    def test_anon_user_cannot_delete_owned_dataset(self):
        user = factories.User()
        owner_org = factories.Organization(
            users=[{'name': user['id'], 'capacity': 'admin'}]
        )
        dataset = factories.Dataset(owner_org=owner_org['id'])

        app = helpers._get_test_app()
        response = app.post(
            url_for(controller='package', action='delete', id=dataset['name']),
            status=403,
        )
        response.mustcontain('Unauthorized to delete package')

        deleted = helpers.call_action('package_show', id=dataset['id'])
        assert_equal('active', deleted['state'])

    def test_logged_in_user_cannot_delete_owned_dataset(self):
        owner = factories.User()
        owner_org = factories.Organization(
            users=[{'name': owner['id'], 'capacity': 'admin'}]
        )
        dataset = factories.Dataset(owner_org=owner_org['id'])

        app = helpers._get_test_app()
        user = factories.User()
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        response = app.post(
            url_for(controller='package', action='delete', id=dataset['name']),
            extra_environ=env,
            expect_errors=True
        )
        assert_equal(403, response.status_int)
        response.mustcontain('Unauthorized to delete package')

    def test_confirm_cancel_delete(self):
        '''Test confirmation of deleting datasets

        When package_delete is made as a get request, it should return a
        'do you want to delete this dataset? confirmation page'''
        user = factories.User()
        owner_org = factories.Organization(
            users=[{'name': user['id'], 'capacity': 'admin'}]
        )
        dataset = factories.Dataset(owner_org=owner_org['id'])

        app = helpers._get_test_app()
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        response = app.get(
            url_for(controller='package', action='delete', id=dataset['name']),
            extra_environ=env,
        )
        assert_equal(200, response.status_int)
        message = 'Are you sure you want to delete dataset - {name}?'
        response.mustcontain(message.format(name=dataset['title']))

        form = response.forms['confirm-dataset-delete-form']
        response = form.submit('cancel')
        response = helpers.webtest_maybe_follow(
            response,
            extra_environ=env,
        )
        assert_equal(200, response.status_int)


class TestResourceNew(helpers.FunctionalTestBase):
    def test_manage_dataset_resource_listing_page(self):
        user = factories.User()
        organization = factories.Organization(user=user)
        dataset = factories.Dataset(owner_org=organization['id'])
        resource = factories.Resource(package_id=dataset['id'])
        app = helpers._get_test_app()
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        response = app.get(
            url_for(
                controller='package',
                action='resources',
                id=dataset['name'],
            ),
            extra_environ=env
        )
        assert_in(resource['name'], response)
        assert_in(resource['description'], response)
        assert_in(resource['format'], response)

    def test_unauth_user_cannot_view_manage_dataset_resource_listing_page(self):
        user = factories.User()
        organization = factories.Organization(user=user)
        dataset = factories.Dataset(owner_org=organization['id'])
        resource = factories.Resource(package_id=dataset['id'])
        app = helpers._get_test_app()
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        response = app.get(
            url_for(
                controller='package',
                action='resources',
                id=dataset['name'],
            ),
            extra_environ=env
        )
        assert_in(resource['name'], response)
        assert_in(resource['description'], response)
        assert_in(resource['format'], response)

    def test_404_on_manage_dataset_resource_listing_page_that_does_not_exist(self):
        user = factories.User()
        app = helpers._get_test_app()
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        response = app.get(
            url_for(
                controller='package',
                action='resources',
                id='does-not-exist'
            ),
            extra_environ=env,
            expect_errors=True
        )
        assert_equal(404, response.status_int)

    def test_add_new_resource_with_link_and_download(self):
        user = factories.User()
        dataset = factories.Dataset()
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        app = helpers._get_test_app()

        response = app.get(
            url_for(
                controller='package',
                action='new_resource',
                id=dataset['id'],
            ),
            extra_environ=env
        )

        form = response.forms['resource-edit']
        form['url'] = u'http://test.com/'
        response = submit_and_follow(app, form, env, 'save',
                                     'go-dataset-complete')

        result = helpers.call_action('package_show', id=dataset['id'])
        response = app.get(
            url_for(
                controller='package',
                action='resource_download',
                id=dataset['id'],
                resource_id=result['resources'][0]['id']
            ),
            extra_environ=env,
        )
        assert_equal(302, response.status_int)

    def test_editor_can_add_new_resource(self):
        user = factories.User()
        organization = factories.Organization(
            users=[{'name': user['id'], 'capacity': 'editor'}]
        )
        dataset = factories.Dataset(
            owner_org=organization['id'],
        )
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        app = helpers._get_test_app()

        response = app.get(
            url_for(
                controller='package',
                action='new_resource',
                id=dataset['id'],
            ),
            extra_environ=env
        )

        form = response.forms['resource-edit']
        form['name'] = u'test resource'
        form['url'] = u'http://test.com/'
        response = submit_and_follow(app, form, env, 'save',
                                     'go-dataset-complete')

        result = helpers.call_action('package_show', id=dataset['id'])
        assert_equal(1, len(result['resources']))
        assert_equal(u'test resource', result['resources'][0]['name'])

    def test_admin_can_add_new_resource(self):
        user = factories.User()
        organization = factories.Organization(
            users=[{'name': user['id'], 'capacity': 'admin'}]
        )
        dataset = factories.Dataset(
            owner_org=organization['id'],
        )
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        app = helpers._get_test_app()

        response = app.get(
            url_for(
                controller='package',
                action='new_resource',
                id=dataset['id'],
            ),
            extra_environ=env
        )

        form = response.forms['resource-edit']
        form['name'] = u'test resource'
        form['url'] = u'http://test.com/'
        response = submit_and_follow(app, form, env, 'save',
                                     'go-dataset-complete')

        result = helpers.call_action('package_show', id=dataset['id'])
        assert_equal(1, len(result['resources']))
        assert_equal(u'test resource', result['resources'][0]['name'])

    def test_member_cannot_add_new_resource(self):
        user = factories.User()
        organization = factories.Organization(
            users=[{'name': user['id'], 'capacity': 'member'}]
        )
        dataset = factories.Dataset(
            owner_org=organization['id'],
        )
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        app = helpers._get_test_app()

        response = app.get(
            url_for(
                controller='package',
                action='new_resource',
                id=dataset['id'],
            ),
            extra_environ=env,
            status=403,
        )

        response = app.post(
            url_for(
                controller='package',
                action='new_resource',
                id=dataset['id'],
            ),
            {'name': 'test', 'url': 'test', 'save': 'save', 'id': ''},
            extra_environ=env,
            status=403,
        )

    def test_non_organization_users_cannot_add_new_resource(self):
        '''on an owned dataset'''
        user = factories.User()
        organization = factories.Organization()
        dataset = factories.Dataset(
            owner_org=organization['id'],
        )
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        app = helpers._get_test_app()

        response = app.get(
            url_for(
                controller='package',
                action='new_resource',
                id=dataset['id'],
            ),
            extra_environ=env,
            status=403,
        )

        response = app.post(
            url_for(
                controller='package',
                action='new_resource',
                id=dataset['id'],
            ),
            {'name': 'test', 'url': 'test', 'save': 'save', 'id': ''},
            extra_environ=env,
            status=403,
        )

    def test_anonymous_users_cannot_add_new_resource(self):
        organization = factories.Organization()
        dataset = factories.Dataset(
            owner_org=organization['id'],
        )
        app = helpers._get_test_app()

        response = app.get(
            url_for(
                controller='package',
                action='new_resource',
                id=dataset['id'],
            ),
            status=403,
        )

        response = app.post(
            url_for(
                controller='package',
                action='new_resource',
                id=dataset['id'],
            ),
            {'name': 'test', 'url': 'test', 'save': 'save', 'id': ''},
            status=403,
        )

    def test_anonymous_users_cannot_edit_resource(self):
        organization = factories.Organization()
        dataset = factories.Dataset(
            owner_org=organization['id'],
        )
        resource = factories.Resource(package_id=dataset['id'])
        app = helpers._get_test_app()

        response = app.get(
            url_for(
                controller='package',
                action='resource_edit',
                id=dataset['id'],
                resource_id=resource['id'],
            ),
            status=403,
        )

        response = app.post(
            url_for(
                controller='package',
                action='resource_edit',
                id=dataset['id'],
                resource_id=resource['id'],
            ),
            {'name': 'test', 'url': 'test', 'save': 'save', 'id': ''},
            status=403,
        )


class TestResourceView(helpers.FunctionalTestBase):
    @classmethod
    def setup_class(cls):
        super(cls, cls).setup_class()

        if not p.plugin_loaded('image_view'):
            p.load('image_view')

        helpers.reset_db()

    @classmethod
    def teardown_class(cls):
        p.unload('image_view')

    def test_existent_resource_view_page_returns_ok_code(self):
        resource_view = factories.ResourceView()

        url = url_for(controller='package',
                      action='resource_read',
                      id=resource_view['package_id'],
                      resource_id=resource_view['resource_id'],
                      view_id=resource_view['id'])

        app = self._get_test_app()
        app.get(url, status=200)

    def test_inexistent_resource_view_page_returns_not_found_code(self):
        resource_view = factories.ResourceView()

        url = url_for(controller='package',
                      action='resource_read',
                      id=resource_view['package_id'],
                      resource_id=resource_view['resource_id'],
                      view_id='inexistent-view-id')

        app = self._get_test_app()
        app.get(url, status=404)

    def test_resource_view_description_is_rendered_as_markdown(self):
        resource_view = factories.ResourceView(description="Some **Markdown**")
        url = url_for(controller='package',
                      action='resource_read',
                      id=resource_view['package_id'],
                      resource_id=resource_view['resource_id'],
                      view_id=resource_view['id'])
        app = self._get_test_app()
        response = app.get(url)
        response.mustcontain('Some <strong>Markdown</strong>')


class TestResourceRead(helpers.FunctionalTestBase):
    def test_existing_resource_with_not_associated_dataset(self):

        dataset = factories.Dataset()
        resource = factories.Resource()

        url = url_for(controller='package',
                      action='resource_read',
                      id=dataset['id'],
                      resource_id=resource['id'])

        app = self._get_test_app()
        app.get(url, status=404)

    def test_resource_read_logged_in_user(self):
        '''
        A logged-in user can view resource page.
        '''
        user = factories.User()
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        dataset = factories.Dataset()
        resource = factories.Resource(package_id=dataset['id'])

        url = url_for(controller='package',
                      action='resource_read',
                      id=dataset['id'],
                      resource_id=resource['id'])

        app = self._get_test_app()
        app.get(url, status=200, extra_environ=env)

    def test_resource_read_anon_user(self):
        '''
        An anon user can view resource page.
        '''
        dataset = factories.Dataset()
        resource = factories.Resource(package_id=dataset['id'])

        url = url_for(controller='package',
                      action='resource_read',
                      id=dataset['id'],
                      resource_id=resource['id'])

        app = self._get_test_app()
        app.get(url, status=200)

    def test_resource_read_sysadmin(self):
        '''
        A sysadmin can view resource page.
        '''
        sysadmin = factories.Sysadmin()
        env = {'REMOTE_USER': sysadmin['name'].encode('ascii')}
        dataset = factories.Dataset()
        resource = factories.Resource(package_id=dataset['id'])

        url = url_for(controller='package',
                      action='resource_read',
                      id=dataset['id'],
                      resource_id=resource['id'])

        app = self._get_test_app()
        app.get(url, status=200, extra_environ=env)

    def test_user_not_in_organization_cannot_read_private_dataset(self):
        user = factories.User()
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        organization = factories.Organization()
        dataset = factories.Dataset(
            owner_org=organization['id'],
            private=True,
        )
        resource = factories.Resource(package_id=dataset['id'])

        url = url_for(controller='package',
                      action='resource_read',
                      id=dataset['id'],
                      resource_id=resource['id'])

        app = self._get_test_app()
        response = app.get(url,
                           status=404,
                           extra_environ=env)

    def test_organization_members_can_read_resources_in_private_datasets(self):
        members = {
            'member': factories.User(),
            'editor': factories.User(),
            'admin': factories.User(),
            'sysadmin': factories.Sysadmin()
        }
        organization = factories.Organization(
            users=[
                {'name': members['member']['id'], 'capacity': 'member'},
                {'name': members['editor']['id'], 'capacity': 'editor'},
                {'name': members['admin']['id'], 'capacity': 'admin'},
            ]
        )
        dataset = factories.Dataset(
            owner_org=organization['id'],
            private=True,
        )
        resource = factories.Resource(package_id=dataset['id'])

        app = helpers._get_test_app()

        for user, user_dict in members.items():
            response = app.get(
                url_for(
                    controller='package',
                    action='resource_read',
                    id=dataset['name'],
                    resource_id=resource['id'],
                ),
                extra_environ={
                    'REMOTE_USER': user_dict['name'].encode('ascii'),
                },
            )
            assert_in('Just another test resource', response.body)

    def test_anonymous_users_cannot_read_private_datasets(self):
        organization = factories.Organization()
        dataset = factories.Dataset(
            owner_org=organization['id'],
            private=True,
        )
        app = helpers._get_test_app()
        response = app.get(
            url_for(controller='package', action='read', id=dataset['name']),
            status=404
        )
        assert_equal(404, response.status_int)


class TestResourceDelete(helpers.FunctionalTestBase):
    def test_dataset_owners_can_delete_resources(self):
        user = factories.User()
        owner_org = factories.Organization(
            users=[{'name': user['id'], 'capacity': 'admin'}]
        )
        dataset = factories.Dataset(owner_org=owner_org['id'])
        resource = factories.Resource(package_id=dataset['id'])
        app = helpers._get_test_app()
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        response = app.post(
            url_for(controller='package', action='resource_delete',
                    id=dataset['name'], resource_id=resource['id']),
            extra_environ=env,
        )
        response = response.follow()
        assert_equal(200, response.status_int)
        response.mustcontain('This dataset has no data')

        assert_raises(p.toolkit.ObjectNotFound, helpers.call_action,
                      'resource_show', id=resource['id'])

    def test_deleting_non_existing_resource_404s(self):
        user = factories.User()
        owner_org = factories.Organization(
            users=[{'name': user['id'], 'capacity': 'admin'}]
        )
        dataset = factories.Dataset(owner_org=owner_org['id'])
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        app = helpers._get_test_app()
        response = app.post(
            url_for(controller='package', action='resource_delete',
                    id=dataset['name'], resource_id='doesnotexist'),
            extra_environ=env,
            expect_errors=True
        )
        assert_equal(404, response.status_int)

    def test_anon_users_cannot_delete_owned_resources(self):
        user = factories.User()
        owner_org = factories.Organization(
            users=[{'name': user['id'], 'capacity': 'admin'}]
        )
        dataset = factories.Dataset(owner_org=owner_org['id'])
        resource = factories.Resource(package_id=dataset['id'])

        app = helpers._get_test_app()
        response = app.post(
            url_for(controller='package', action='resource_delete',
                    id=dataset['name'], resource_id=resource['id']),
            status=403,
        )
        response.mustcontain('Unauthorized to delete package')

    def test_logged_in_users_cannot_delete_resources_they_do_not_own(self):
        # setup our dataset
        owner = factories.User()
        owner_org = factories.Organization(
            users=[{'name': owner['id'], 'capacity': 'admin'}]
        )
        dataset = factories.Dataset(owner_org=owner_org['id'])
        resource = factories.Resource(package_id=dataset['id'])

        # access as another user
        user = factories.User()
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        app = helpers._get_test_app()
        response = app.post(
            url_for(controller='package', action='resource_delete',
                    id=dataset['name'], resource_id=resource['id']),
            extra_environ=env,
            expect_errors=True
        )
        assert_equal(403, response.status_int)
        response.mustcontain('Unauthorized to delete package')

    def test_sysadmins_can_delete_any_resource(self):
        owner_org = factories.Organization()
        dataset = factories.Dataset(owner_org=owner_org['id'])
        resource = factories.Resource(package_id=dataset['id'])

        sysadmin = factories.Sysadmin()
        app = helpers._get_test_app()
        env = {'REMOTE_USER': sysadmin['name'].encode('ascii')}
        response = app.post(
            url_for(controller='package', action='resource_delete',
                    id=dataset['name'], resource_id=resource['id']),
            extra_environ=env,
        )
        response = response.follow()
        assert_equal(200, response.status_int)
        response.mustcontain('This dataset has no data')

        assert_raises(p.toolkit.ObjectNotFound, helpers.call_action,
                      'resource_show', id=resource['id'])

    def test_confirm_and_cancel_deleting_a_resource(self):
        '''Test confirmation of deleting resources

        When resource_delete is made as a get request, it should return a
        'do you want to delete this reource? confirmation page'''
        user = factories.User()
        owner_org = factories.Organization(
            users=[{'name': user['id'], 'capacity': 'admin'}]
        )
        dataset = factories.Dataset(owner_org=owner_org['id'])
        resource = factories.Resource(package_id=dataset['id'])
        app = helpers._get_test_app()
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        response = app.get(
            url_for(controller='package', action='resource_delete',
                    id=dataset['name'], resource_id=resource['id']),
            extra_environ=env,
        )
        assert_equal(200, response.status_int)
        message = 'Are you sure you want to delete resource - {name}?'
        response.mustcontain(message.format(name=resource['name']))

        # cancelling sends us back to the resource edit page
        form = response.forms['confirm-resource-delete-form']
        response = form.submit('cancel')
        response = response.follow(extra_environ=env)
        assert_equal(200, response.status_int)


class TestSearch(helpers.FunctionalTestBase):
    def test_search_basic(self):
        dataset1 = factories.Dataset()

        offset = url_for(controller='package', action='search')
        app = self._get_test_app()
        page = app.get(offset)

        assert dataset1['name'] in page.body.decode('utf8')

    def test_search_language_toggle(self):
        dataset1 = factories.Dataset()

        offset = url_for(controller='package', action='search', q=dataset1['name'])
        app = self._get_test_app()
        page = app.get(offset)

        assert dataset1['name'] in page.body.decode('utf8')
        assert ('q=' + dataset1['name']) in page.body.decode('utf8')

    def test_search_sort_by_blank(self):
        factories.Dataset()

        # ?sort has caused an exception in the past
        offset = url_for(controller='package', action='search') + '?sort'
        app = self._get_test_app()
        app.get(offset)

    def test_search_sort_by_bad(self):
        factories.Dataset()

        # bad spiders try all sorts of invalid values for sort. They should get
        # a 400 error with specific error message. No need to alert the
        # administrator.
        offset = url_for(controller='package', action='search') + \
            '?sort=gvgyr_fgevat+nfp'
        app = self._get_test_app()
        response = app.get(offset, status=[200, 400])
        if response.status == 200:
            import sys
            sys.stdout.write(response.body)
            raise Exception("Solr returned an unknown error message. "
                            "Please check the error handling "
                            "in ckan/lib/search/query.py:run")

    def test_search_solr_syntax_error(self):
        factories.Dataset()

        # SOLR raises SyntaxError when it can't parse q (or other fields?).
        # Whilst this could be due to a bad user input, it could also be
        # because CKAN mangled things somehow and therefore we flag it up to
        # the administrator and give a meaningless error, just in case
        offset = url_for(controller='package', action='search') + \
            '?q=--included'
        app = self._get_test_app()
        search_response = app.get(offset)

        search_response_html = BeautifulSoup(search_response.body)
        err_msg = search_response_html.select('#search-error')
        err_msg = ''.join([n.text for n in err_msg])
        assert_in('error while searching', err_msg)

    def test_search_plugin_hooks(self):
        with p.use_plugin('test_package_controller_plugin') as plugin:

            offset = url_for(controller='package', action='search')
            app = self._get_test_app()
            app.get(offset)

            # get redirected ...
            assert plugin.calls['before_search'] == 1, plugin.calls
            assert plugin.calls['after_search'] == 1, plugin.calls

    def test_search_page_request(self):
        '''Requesting package search page returns list of datasets.'''
        app = self._get_test_app()
        factories.Dataset(name="dataset-one", title='Dataset One')
        factories.Dataset(name="dataset-two", title='Dataset Two')
        factories.Dataset(name="dataset-three", title='Dataset Three')

        search_url = url_for(controller='package', action='search')
        search_response = app.get(search_url)

        assert_true('3 datasets found' in search_response)

        search_response_html = BeautifulSoup(search_response.body)
        ds_titles = search_response_html.select('.dataset-list '
                                                '.dataset-item '
                                                '.dataset-heading a')
        ds_titles = [n.string for n in ds_titles]

        assert_equal(len(ds_titles), 3)
        assert_true('Dataset One' in ds_titles)
        assert_true('Dataset Two' in ds_titles)
        assert_true('Dataset Three' in ds_titles)

    def test_search_page_results(self):
        '''Searching for datasets returns expected results.'''
        app = self._get_test_app()
        factories.Dataset(name="dataset-one", title='Dataset One')
        factories.Dataset(name="dataset-two", title='Dataset Two')
        factories.Dataset(name="dataset-three", title='Dataset Three')

        search_url = url_for(controller='package', action='search')
        search_response = app.get(search_url)

        search_form = search_response.forms['dataset-search-form']
        search_form['q'] = 'One'
        search_results = webtest_submit(search_form)

        assert_true('1 dataset found' in search_results)

        search_response_html = BeautifulSoup(search_results.body)
        ds_titles = search_response_html.select('.dataset-list '
                                                '.dataset-item '
                                                '.dataset-heading a')
        ds_titles = [n.string for n in ds_titles]

        assert_equal(len(ds_titles), 1)
        assert_true('Dataset One' in ds_titles)

    def test_search_page_no_results(self):
        '''Search with non-returning phrase returns no results.'''
        app = self._get_test_app()
        factories.Dataset(name="dataset-one", title='Dataset One')
        factories.Dataset(name="dataset-two", title='Dataset Two')
        factories.Dataset(name="dataset-three", title='Dataset Three')

        search_url = url_for(controller='package', action='search')
        search_response = app.get(search_url)

        search_form = search_response.forms['dataset-search-form']
        search_form['q'] = 'Nout'
        search_results = webtest_submit(search_form)

        assert_true('No datasets found for &#34;Nout&#34;' in search_results)

        search_response_html = BeautifulSoup(search_results.body)
        ds_titles = search_response_html.select('.dataset-list '
                                                '.dataset-item '
                                                '.dataset-heading a')
        ds_titles = [n.string for n in ds_titles]

        assert_equal(len(ds_titles), 0)

    def test_search_page_results_tag(self):
        '''Searching with a tag returns expected results.'''
        app = self._get_test_app()
        factories.Dataset(name="dataset-one", title='Dataset One',
                          tags=[{'name': 'my-tag'}])
        factories.Dataset(name="dataset-two", title='Dataset Two')
        factories.Dataset(name="dataset-three", title='Dataset Three')

        search_url = url_for(controller='package', action='search')
        search_response = app.get(search_url)

        assert_true('/dataset?tags=my-tag' in search_response)

        tag_search_response = app.get('/dataset?tags=my-tag')

        assert_true('1 dataset found' in tag_search_response)

        search_response_html = BeautifulSoup(tag_search_response.body)
        ds_titles = search_response_html.select('.dataset-list '
                                                '.dataset-item '
                                                '.dataset-heading a')
        ds_titles = [n.string for n in ds_titles]

        assert_equal(len(ds_titles), 1)
        assert_true('Dataset One' in ds_titles)

    def test_search_page_results_private(self):
        '''Private datasets don't show up in dataset search results.'''
        app = self._get_test_app()
        org = factories.Organization()

        factories.Dataset(name="dataset-one", title='Dataset One',
                          owner_org=org['id'], private=True)
        factories.Dataset(name="dataset-two", title='Dataset Two')
        factories.Dataset(name="dataset-three", title='Dataset Three')

        search_url = url_for(controller='package', action='search')
        search_response = app.get(search_url)

        search_response_html = BeautifulSoup(search_response.body)
        ds_titles = search_response_html.select('.dataset-list '
                                                '.dataset-item '
                                                '.dataset-heading a')
        ds_titles = [n.string for n in ds_titles]

        assert_equal(len(ds_titles), 2)
        assert_true('Dataset One' not in ds_titles)
        assert_true('Dataset Two' in ds_titles)
        assert_true('Dataset Three' in ds_titles)

    def test_user_not_in_organization_cannot_search_private_datasets(self):
        app = helpers._get_test_app()
        user = factories.User()
        organization = factories.Organization()
        dataset = factories.Dataset(
            owner_org=organization['id'],
            private=True,
        )
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        search_url = url_for(controller='package', action='search')
        search_response = app.get(search_url, extra_environ=env)

        search_response_html = BeautifulSoup(search_response.body)
        ds_titles = search_response_html.select('.dataset-list '
                                                '.dataset-item '
                                                '.dataset-heading a')
        assert_equal([n.string for n in ds_titles], [])

    def test_user_in_organization_can_search_private_datasets(self):
        app = helpers._get_test_app()
        user = factories.User()
        organization = factories.Organization(
            users=[{'name': user['id'], 'capacity': 'member'}])
        dataset = factories.Dataset(
            title='A private dataset',
            owner_org=organization['id'],
            private=True,
        )
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        search_url = url_for(controller='package', action='search')
        search_response = app.get(search_url, extra_environ=env)

        search_response_html = BeautifulSoup(search_response.body)
        ds_titles = search_response_html.select('.dataset-list '
                                                '.dataset-item '
                                                '.dataset-heading a')
        assert_equal([n.string for n in ds_titles], ['A private dataset'])

    def test_user_in_different_organization_cannot_search_private_datasets(self):
        app = helpers._get_test_app()
        user = factories.User()
        org1 = factories.Organization(
            users=[{'name': user['id'], 'capacity': 'member'}])
        org2 = factories.Organization()
        dataset = factories.Dataset(
            title='A private dataset',
            owner_org=org2['id'],
            private=True,
        )
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        search_url = url_for(controller='package', action='search')
        search_response = app.get(search_url, extra_environ=env)

        search_response_html = BeautifulSoup(search_response.body)
        ds_titles = search_response_html.select('.dataset-list '
                                                '.dataset-item '
                                                '.dataset-heading a')
        assert_equal([n.string for n in ds_titles], [])

    @helpers.change_config('ckan.search.default_include_private', 'false')
    def test_search_default_include_private_false(self):
        app = helpers._get_test_app()
        user = factories.User()
        organization = factories.Organization(
            users=[{'name': user['id'], 'capacity': 'member'}])
        dataset = factories.Dataset(
            owner_org=organization['id'],
            private=True,
        )
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        search_url = url_for(controller='package', action='search')
        search_response = app.get(search_url, extra_environ=env)

        search_response_html = BeautifulSoup(search_response.body)
        ds_titles = search_response_html.select('.dataset-list '
                                                '.dataset-item '
                                                '.dataset-heading a')
        assert_equal([n.string for n in ds_titles], [])

    def test_sysadmin_can_search_private_datasets(self):
        app = helpers._get_test_app()
        user = factories.Sysadmin()
        organization = factories.Organization()
        dataset = factories.Dataset(
            title='A private dataset',
            owner_org=organization['id'],
            private=True,
        )
        env = {'REMOTE_USER': user['name'].encode('ascii')}
        search_url = url_for(controller='package', action='search')
        search_response = app.get(search_url, extra_environ=env)

        search_response_html = BeautifulSoup(search_response.body)
        ds_titles = search_response_html.select('.dataset-list '
                                                '.dataset-item '
                                                '.dataset-heading a')
        assert_equal([n.string for n in ds_titles], ['A private dataset'])


class TestPackageFollow(helpers.FunctionalTestBase):

    def test_package_follow(self):
        app = self._get_test_app()

        user = factories.User()
        package = factories.Dataset()

        env = {'REMOTE_USER': user['name'].encode('ascii')}
        follow_url = url_for(controller='package',
                             action='follow',
                             id=package['id'])
        response = app.post(follow_url, extra_environ=env, status=302)
        response = response.follow()
        assert_true('You are now following {0}'
                    .format(package['title'])
                    in response)

    def test_package_follow_not_exist(self):
        '''Pass an id for a package that doesn't exist'''
        app = self._get_test_app()

        user_one = factories.User()

        env = {'REMOTE_USER': user_one['name'].encode('ascii')}
        follow_url = url_for(controller='package',
                             action='follow',
                             id='not-here')
        response = app.post(follow_url, extra_environ=env, status=302)
        response = response.follow(status=404)
        assert_true('Dataset not found' in response)

    def test_package_unfollow(self):
        app = self._get_test_app()

        user_one = factories.User()
        package = factories.Dataset()

        env = {'REMOTE_USER': user_one['name'].encode('ascii')}
        follow_url = url_for(controller='package',
                             action='follow',
                             id=package['id'])
        app.post(follow_url, extra_environ=env, status=302)

        unfollow_url = url_for(controller='package', action='unfollow',
                               id=package['id'])
        unfollow_response = app.post(unfollow_url, extra_environ=env,
                                     status=302)
        unfollow_response = unfollow_response.follow()

        assert_true('You are no longer following {0}'
                    .format(package['title'])
                    in unfollow_response)

    def test_package_unfollow_not_following(self):
        '''Unfollow a package not currently following'''
        app = self._get_test_app()

        user_one = factories.User()
        package = factories.Dataset()

        env = {'REMOTE_USER': user_one['name'].encode('ascii')}
        unfollow_url = url_for(controller='package', action='unfollow',
                               id=package['id'])
        unfollow_response = app.post(unfollow_url, extra_environ=env,
                                     status=302)
        unfollow_response = unfollow_response.follow()

        assert_true('You are not following {0}'.format(package['id'])
                    in unfollow_response)

    def test_package_unfollow_not_exist(self):
        '''Unfollow a package that doesn't exist.'''
        app = self._get_test_app()

        user_one = factories.User()

        env = {'REMOTE_USER': user_one['name'].encode('ascii')}
        unfollow_url = url_for(controller='package', action='unfollow',
                               id='not-here')
        unfollow_response = app.post(unfollow_url, extra_environ=env,
                                     status=302)
        unfollow_response = unfollow_response.follow(status=404)
        assert_true('Dataset not found' in unfollow_response)

    def test_package_follower_list(self):
        '''Following users appear on followers list page.'''
        app = self._get_test_app()

        user_one = factories.Sysadmin()
        package = factories.Dataset()

        env = {'REMOTE_USER': user_one['name'].encode('ascii')}
        follow_url = url_for(controller='package',
                             action='follow',
                             id=package['id'])
        app.post(follow_url, extra_environ=env, status=302)

        followers_url = url_for(controller='package', action='followers',
                                id=package['id'])

        # Only sysadmins can view the followers list pages
        followers_response = app.get(followers_url, extra_environ=env,
                                     status=200)
        assert_true(user_one['display_name'] in followers_response)


class TestDatasetRead(helpers.FunctionalTestBase):

    def test_dataset_read(self):
        app = self._get_test_app()

        dataset = factories.Dataset()

        url = url_for(controller='package',
                      action='read',
                      id=dataset['id'])
        response = app.get(url)
        assert_in(dataset['title'], response)



# encoding: utf-8

import os
from nose import tools as nosetools

from ckan.common import config

import ckan.tests.helpers as h
import ckan.plugins as p
from ckan.config import environment
from ckan.exceptions import CkanConfigurationException

from ckan.tests import helpers


class TestUpdateConfig(h.FunctionalTestBase):

    '''
    Tests for config settings from env vars, set in
    config.environment.update_config().
    '''

    ENV_VAR_LIST = [
        ('CKAN_SQLALCHEMY_URL', 'postgresql://mynewsqlurl/'),
        ('CKAN_DATASTORE_WRITE_URL', 'http://mynewdbwriteurl/'),
        ('CKAN_DATASTORE_READ_URL', 'http://mynewdbreadurl/'),
        ('CKAN_SOLR_URL', 'http://mynewsolrurl/solr'),
        ('CKAN_SITE_ID', 'my-site'),
        ('CKAN_DB', 'postgresql://mydeprectatesqlurl/'),
        ('CKAN_SMTP_SERVER', 'mail.example.com'),
        ('CKAN_SMTP_STARTTLS', 'True'),
        ('CKAN_SMTP_USER', 'my_user'),
        ('CKAN_SMTP_PASSWORD', 'password'),
        ('CKAN_SMTP_MAIL_FROM', 'server@example.com')
    ]

    def _setup_env_vars(self):
        for env_var, value in self.ENV_VAR_LIST:
            os.environ.setdefault(env_var, value)
        # plugin.load() will force the config to update
        p.load()

    def teardown(self):
        for env_var, _ in self.ENV_VAR_LIST:
            if os.environ.get(env_var, None):
                del os.environ[env_var]
        # plugin.load() will force the config to update
        p.load()

    def test_update_config_env_vars(self):
        '''
        Setting an env var from the whitelist will set the appropriate option
        in config object.
        '''
        self._setup_env_vars()

        nosetools.assert_equal(config['solr_url'], 'http://mynewsolrurl/solr')
        nosetools.assert_equal(config['sqlalchemy.url'],
                               'postgresql://mynewsqlurl/')
        nosetools.assert_equal(config['ckan.datastore.write_url'],
                               'http://mynewdbwriteurl/')
        nosetools.assert_equal(config['ckan.datastore.read_url'],
                               'http://mynewdbreadurl/')
        nosetools.assert_equal(config['ckan.site_id'], 'my-site')
        nosetools.assert_equal(config['smtp.server'], 'mail.example.com')
        nosetools.assert_equal(config['smtp.starttls'], 'True')
        nosetools.assert_equal(config['smtp.user'], 'my_user')
        nosetools.assert_equal(config['smtp.password'], 'password')
        nosetools.assert_equal(config['smtp.mail_from'], 'server@example.com')

    def test_update_config_db_url_precedence(self):
        '''CKAN_SQLALCHEMY_URL in the env takes precedence over CKAN_DB'''
        os.environ.setdefault('CKAN_DB', 'postgresql://mydeprectatesqlurl/')
        os.environ.setdefault('CKAN_SQLALCHEMY_URL',
                              'postgresql://mynewsqlurl/')
        p.load()

        nosetools.assert_equal(config['sqlalchemy.url'],
                               'postgresql://mynewsqlurl/')


class TestSiteUrlMandatory(object):

    @helpers.change_config('ckan.site_url', '')
    def test_missing_siteurl(self):
        nosetools.assert_raises(RuntimeError, environment.update_config)

    @helpers.change_config('ckan.site_url', 'demo.ckan.org')
    def test_siteurl_missing_schema(self):
        nosetools.assert_raises(RuntimeError, environment.update_config)

    @helpers.change_config('ckan.site_url', 'ftp://demo.ckan.org')
    def test_siteurl_wrong_schema(self):
        nosetools.assert_raises(RuntimeError, environment.update_config)

    @helpers.change_config('ckan.site_url', 'http://demo.ckan.org/')
    def test_siteurl_removes_backslash(self):
        environment.update_config()
        nosetools.assert_equals(config['ckan.site_url'],
                                'http://demo.ckan.org')


class TestDisplayTimezone(object):

    @helpers.change_config('ckan.display_timezone', 'Krypton/Argo City')
    def test_missing_timezone(self):
        nosetools.assert_raises(CkanConfigurationException, environment.update_config)



# encoding: utf-8

import math
import os

from ckan.common import config
from repoze.who.plugins import auth_tkt as repoze_auth_tkt

_bool = repoze_auth_tkt._bool

import logging
log = logging.getLogger(__name__)


class CkanAuthTktCookiePlugin(repoze_auth_tkt.AuthTktCookiePlugin):

    def __init__(self, httponly, *args, **kwargs):
        super(CkanAuthTktCookiePlugin, self).__init__(*args, **kwargs)
        self.httponly = httponly

    def _get_cookies(self, *args, **kwargs):
        '''
        Override method in superclass to ensure HttpOnly is set appropriately.
        '''
        super_cookies = super(CkanAuthTktCookiePlugin, self). \
            _get_cookies(*args, **kwargs)

        cookies = []
        for k, v in super_cookies:
            replace_with = '; HttpOnly' if self.httponly else ''
            v = v.replace('; HttpOnly', '') + replace_with
            cookies.append((k, v))

        return cookies


def make_plugin(secret=None,
                secretfile=None,
                cookie_name='auth_tkt',
                secure=False,
                include_ip=False,
                timeout=None,
                reissue_time=None,
                userid_checker=None):
    from repoze.who.utils import resolveDotted

    # ckan specifics:
    # Get secret from beaker setting if necessary
    if secret is None or secret == 'somesecret':
        secret = config['beaker.session.secret']
    # Session timeout and reissue time for auth cookie
    if timeout is None and config.get('who.timeout'):
        timeout = config.get('who.timeout')
    if reissue_time is None and config.get('who.reissue_time'):
        reissue_time = config.get('who.reissue_time')
    if timeout is not None and reissue_time is None:
        reissue_time = int(math.ceil(int(timeout) * 0.1))
    # Set httponly based on config value. Default is True
    httponly = config.get('who.httponly', True)
    # Set secure based on config value. Default is False
    secure = config.get('who.secure', False)

    # back to repoze boilerplate
    if (secret is None and secretfile is None):
        raise ValueError("One of 'secret' or 'secretfile' must not be None.")
    if (secret is not None and secretfile is not None):
        raise ValueError("Specify only one of 'secret' or 'secretfile'.")
    if secretfile:
        secretfile = os.path.abspath(os.path.expanduser(secretfile))
        if not os.path.exists(secretfile):
            raise ValueError("No such 'secretfile': %s" % secretfile)
        secret = open(secretfile).read().strip()
    if timeout:
        timeout = int(timeout)
    if reissue_time:
        reissue_time = int(reissue_time)
    if userid_checker is not None:
        userid_checker = resolveDotted(userid_checker)
    plugin = CkanAuthTktCookiePlugin(_bool(httponly),
                                     secret,
                                     cookie_name,
                                     _bool(secure),
                                     _bool(include_ip),
                                     timeout,
                                     reissue_time,
                                     userid_checker)
    return plugin



# encoding: utf-8

from nose.plugins import Plugin
from inspect import isclass
import hashlib
import os
import sys
import re
import pkg_resources
from paste.deploy import loadapp
from ckan.common import config
import unittest
import time

class CkanNose(Plugin):
    settings = None

    def startContext(self, ctx):
        # import needs to be here or setup happens too early
        import ckan.model as model

        if 'legacy' not in repr(ctx):
            # We don't want to do the stuff below for new-style tests.
            if not CkanNose.settings.reset_database:
                model.repo.tables_created_and_initialised = True
            return

        if isclass(ctx):
            if hasattr(ctx, "no_db") and ctx.no_db:
                return
            if (not CkanNose.settings.reset_database
                    and not CkanNose.settings.ckan_migration):
                model.Session.close_all()
                model.repo.tables_created_and_initialised = True
                model.repo.rebuild_db()
                self.is_first_test = False
            elif self.is_first_test or CkanNose.settings.ckan_migration:
                model.Session.close_all()
                model.repo.clean_db()
                self.is_first_test = False
                if CkanNose.settings.ckan_migration:
                    model.Session.close_all()
                    model.repo.upgrade_db()

            ## This is to make sure the configuration is run again.
            ## Plugins use configure to make their own tables and they
            ## may need to be recreated to make tests work.
            from ckan.plugins import PluginImplementations
            from ckan.plugins.interfaces import IConfigurable
            for plugin in PluginImplementations(IConfigurable):
                plugin.configure(config)

            # init_db is run at the start of every class because
            # when you use an in-memory sqlite db, it appears that
            # the db is destroyed after every test when you Session.Remove().
            model.repo.init_db()

    def options(self, parser, env):
        parser.add_option(
            '--ckan',
            action='store_true',
            dest='is_ckan',
            help='Always set this when testing CKAN.')
        parser.add_option(
            '--ckan-migration',
            action='store_true',
            dest='ckan_migration',
            help='set this when wanting to test migrations')
        parser.add_option(
            '--docstrings',
            action='store_true',
            dest='docstrings',
            help='set this to display test docstrings instead of module names')
        parser.add_option(
            '--segments',
            dest='segments',
            help='A string containing a hex digits that represent which of'
                 'the 16 test segments to run. i.e 15af will run segments 1,5,a,f')
        parser.add_option(
            '--reset-db',
            action='store_true',
            dest='reset_database',
            help='drop database and reinitialize before tests are run')

    def wantClass(self, cls):
        name = cls.__name__

        wanted = (not cls.__name__.startswith('_')
                  and (issubclass(cls, unittest.TestCase)
                       or re.search('(?:^|[\b_\./-])[Tt]est', name)
                      ))
        
        if self.segments and str(hashlib.md5(name).hexdigest())[0] not in self.segments:
            return False

        return wanted

    def finalize(self, report):
        if self.segments:
            print 'Segments: %s' % self.segments

    def configure(self, settings, config):
        CkanNose.settings = settings
        if settings.is_ckan:
            self.enabled = True
            self.is_first_test = True
        self.segments = settings.segments

    def describeTest(self, test):
        if not CkanNose.settings.docstrings:
            # display module name instead of docstring
            return False

    def startTest(self, test):
        """
        startTest: start timing.
        """
##        self._started = time.time()

    def stopTest(self, test):
        """
        stopTest: stop timing, canonicalize the test name, and save
        the run time.
        """
##        runtime = time.time() - self._started
##
##        # CTB: HACK!
##        f = open('times.txt', 'a')
##
##        testname = str(test)
##        #if ' ' in testname:
##        #    testname = testname.split()[1]
##
##        f.write('%s,%s\n' % (testname, str(runtime)))
##
##        f.close()



# encoding: utf-8

from nose.tools import assert_equal

from ckan import model
from ckan.lib.create_test_data import CreateTestData
from ckan.lib.search.sql import PackageSearchQuery

class TestSimpleSearch:
    @classmethod
    def setup_class(cls):
        CreateTestData.create()
    
    @classmethod
    def teardown_class(cls):
        model.repo.rebuild_db()

    def test_get_all_entity_ids(self):
        ids = PackageSearchQuery().get_all_entity_ids()
        anna = model.Package.by_name(u'annakarenina')
        assert anna.id in ids
        assert len(ids) >= 2, len(ids)
        
    def test_run_query_basic(self):
        res = PackageSearchQuery().run({'q':'annakarenina'})
        anna = model.Package.by_name(u'annakarenina')
        assert_equal(res, {'results': [{'id': anna.id}], 'count': 1})

    def test_run_query_home(self):
        # This is the query from the CKAN home page
        res = PackageSearchQuery().run({'q': '*:*'})
        assert res['count'] >= 2, res['count']

    def test_run_query_all(self):
        # This is the default query from the search page
        res = PackageSearchQuery().run({'q': u''})
        assert res['count'] >= 2, res['count']



# encoding: utf-8

from ckan.common import config

import ckan.plugins as plugins
import ckan.plugins.toolkit as toolkit


def package_delete(context, data_dict=None):
    return {'success': False,
            'msg': 'Only sysadmins can delete packages'}


class ExampleIAuthFunctionsPlugin(plugins.SingletonPlugin):
    plugins.implements(plugins.IAuthFunctions)

    def get_auth_functions(self):
        return {'package_delete': package_delete}



# encoding: utf-8

import os.path
import sys
import logging
import ConfigParser

from fanstatic import Library, Resource, Group, get_library_registry
import fanstatic.core as core

# This imports patches fanstatic
import ckan.lib.fanstatic_extensions as fanstatic_extensions

log = logging.getLogger(__name__)


def min_path(path):
    '''Return the .min.* filename for the given .js or .css file.

    For example moo.js -> moo.min.js

    '''
    path, ext = os.path.splitext(path)
    return path + '.min' + ext


def create_library(name, path, depend_base=True):
    ''' Creates a fanstatic library `name` with the contents of a
    directory `path` using resource.config if found.'''

    def get_resource(lib_name, resource_name):
        ''' Attempt to get the resource from the current lib or if not try
        assume it is a fully qualified resource name. '''
        try:
            res = getattr(module, '%s/%s' % (lib_name, resource_name))
        except AttributeError:
            res = getattr(module, '%s' % resource_name)
        return res

    def create_resource(path, lib_name, count, inline=False):
        ''' create the fanstatic Resource '''
        renderer = None
        kw = {}
        if not inline:
            # resource_name is name of the file without the .js/.css
            rel_path, filename = os.path.split(path)
            filename = os.path.join(rel_path, filename)
            path_min = min_path(os.path.join(resource_path, filename))
            if os.path.exists(path_min):
                kw['minified'] = min_path(filename)
            if filename.endswith('.js'):
                renderer = core.render_js
                if path not in force_top:
                    kw['bottom'] = True
            if filename.endswith('.css'):
                renderer = core.render_css
            core.set_resource_file_existence_checking(True)
        else:
            # This doesn't exist so stop fanstatic checking the filesystem
            if path not in force_top:
                kw['bottom'] = True
            core.set_resource_file_existence_checking(False)
        dependencies = []
        if path in depends:
            for dependency in depends[path]:
                dependencies.append(get_resource(name, dependency))
        if depend_base:
            dependencies.append(getattr(module, 'base/main'))
        if dependencies:
            kw['depends'] = dependencies
        if path in dont_bundle:
            kw['dont_bundle'] = True
        # IE conditionals
        condition = None
        other_browsers = False
        if path in IE_conditionals:
            other_browsers = ('others' in IE_conditionals[path])
            condition = IE_conditionals[path][0]
        if inline or condition:
            kw['renderer'] = fanstatic_extensions.CkanCustomRenderer(
                                        condition=condition,
                                        script=inline,
                                        renderer=renderer,
                                        other_browsers=other_browsers)
        resource = Resource(library, path, **kw)

        # Add our customised ordering
        if path in custom_render_order:
            resource.order = custom_render_order[path]
        resource.custom_order = count
        # Update the attributes of the minified version of the resource to
        # that of the parents as fanstatic does not pass these on.
        update_attributes = ['custom_order', 'order', 'bottom', 'depends',
                             'dont_bundle', 'renderer']
        if 'minified' in resource.modes:
            min_res = resource.modes['minified']
            for attribute in update_attributes:
                setattr(min_res, attribute, getattr(resource, attribute))

        # add the resource to this module
        fanstatic_name = '%s/%s' % (lib_name, path)
        setattr(module, fanstatic_name, resource)
        return resource

    resource_path = os.path.join(os.path.dirname(__file__), path)
    library = Library(name, path)
    module = sys.modules[__name__]

    # config options
    order = []
    dont_bundle = []
    force_top = []
    depends = {}
    groups = {}
    IE_conditionals = {}
    custom_render_order = {}
    inline_scripts = {}

    # parse the resource.config file if it exists
    config_path = os.path.join(resource_path, 'resource.config')
    if os.path.exists(config_path):
        config = ConfigParser.RawConfigParser()
        config.read(config_path)

        if config.has_option('main', 'order'):
            order = config.get('main', 'order').split()
        if config.has_option('main', 'dont_bundle'):
            dont_bundle = config.get('main', 'dont_bundle').split()
        if config.has_option('main', 'force_top'):
            force_top = config.get('main', 'force_top').split()

        if config.has_section('depends'):
            items = config.items('depends')
            depends = dict((n, v.split()) for (n, v) in items)
        if config.has_section('groups'):
            items = config.items('groups')
            groups = dict((n, v.split()) for (n, v) in items)
        if config.has_section('custom render order'):
            items = config.items('custom render order')
            custom_render_order = dict((n, int(v)) for (n, v) in items)
        if config.has_section('inline scripts'):
            items = config.items('inline scripts')
            inline_scripts = dict((n, v) for (n, v) in items)
        if config.has_section('IE conditional'):
            items = config.items('IE conditional')
            for (n, v) in items:
                files = v.split()
                for f in files:
                    if f not in IE_conditionals:
                        IE_conditionals[f] = []
                    IE_conditionals[f].append(n)

    # add dependencies for resources in groups
    for group in groups:
        if group in depends:
            for resource in groups[group]:
                if resource not in depends:
                    depends[resource] = []
                for dep in depends[group]:
                    if dep not in depends[resource]:
                        depends[resource].append(dep)

    # process each .js/.css file found
    resource_list = []
    for dirname, dirnames, filenames in os.walk(resource_path):
        for f in filenames:
            rel_path = dirname[len(path):]
            if rel_path:
                rel_path = rel_path[1:]
            filepath = os.path.join(rel_path, f)
            filename_only, extension = os.path.splitext(f)
            if extension in ('.css', '.js') and (
                not filename_only.endswith('.min')):
              resource_list.append(filepath)

    # if groups are defined make sure the order supplied there is honored
    for group in groups:
        for resource in groups[group]:
            if resource not in order:
                # make sure any dependencies are met when we get to creating
                # the resource
                if resource in depends:
                    for dep in depends[resource]:
                        if dep not in order:
                            order.append(dep)
                order.append(resource)

    # add inline scripts
    for inline in inline_scripts:
        resource_list.append(inline)
        if inline not in custom_render_order:
            custom_render_order[inline] = 20

    # order resource_list so that resources are created in the correct order
    for resource_name in reversed(order):
        if resource_name in resource_list:
            resource_list.remove(resource_name)
            resource_list.insert(0, resource_name)

    # create the resources and keep them ordered as we define them.
    count = 0
    for resource_name in resource_list:
        if resource_name in inline_scripts:
            inline = inline_scripts[resource_name].strip()
        else:
            inline = None
        create_resource(resource_name, name, count, inline=inline)
        count += 1

    # add groups
    for group_name in groups:
        members = []
        for member in groups[group_name]:
            fanstatic_name = '%s/%s' % (name, member)
            members.append(getattr(module, fanstatic_name))
        group = Group(members)
        fanstatic_name = '%s/%s' % (name, group_name)
        setattr(module, fanstatic_name, group)

    # finally add the library to this module
    setattr(module, name, library)
    # add to fanstatic
    registry = get_library_registry()
    registry.add(library)

base_path = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                         '..', 'public', 'base'))

create_library('vendor', os.path.join(base_path, 'vendor'), depend_base=False)

create_library('base', os.path.join(base_path, 'javascript'),
               depend_base=False)

create_library('datapreview', os.path.join(base_path, 'datapreview'),
               depend_base=False)

create_library('css', os.path.join(base_path, 'css'), depend_base=False)



# encoding: utf-8

import math
import string

from nose.tools import assert_equal, assert_true, assert_false, assert_in
from bs4 import BeautifulSoup

from routes import url_for

import ckan.tests.helpers as helpers
from ckan.tests import factories

webtest_submit = helpers.webtest_submit
submit_and_follow = helpers.submit_and_follow


def _make_tag_list(n=26):
    '''Returns a list of tag dicts, starting with 'aa, bb, ..., zz', then
    'aaa, bbb, ..., zzz', etc. Tags must be at least 2 characters.'''
    lc = string.lowercase
    lc_len = len(lc)
    return [{'name': lc[i % lc_len] * int(math.ceil(i / lc_len) + 2)}
            for i in range(0, n)]


class TestTagIndex(helpers.FunctionalTestBase):

    def test_tags_listed_under_50(self):
        '''Tag index lists tags under 50 tags.'''
        app = self._get_test_app()
        expected_tags = _make_tag_list(49)
        factories.Dataset(tags=expected_tags)

        tag_index_url = url_for(controller='tag', action='index')
        tag_response = app.get(tag_index_url)

        tag_response_html = BeautifulSoup(tag_response.body)
        tags = [t.string for t in tag_response_html.select('.tag')]

        expected_tag_values = [t['name'] for t in expected_tags]

        assert_equal(len(tags), 49)
        for t in expected_tag_values:
            assert_true(t in tags)

        # no pagination
        assert_false(tag_response_html.select('.pagination'))

    def test_tags_listed_over_50(self):
        '''Tag index lists tags over 50 tags.'''
        app = self._get_test_app()
        expected_tags = _make_tag_list(51)
        factories.Dataset(tags=expected_tags)

        tag_index_url = url_for(controller='tag', action='index')
        tag_response = app.get(tag_index_url)

        tag_response_html = BeautifulSoup(tag_response.body)
        tags = [t.string for t in tag_response_html.select('.tag')]

        expected_tag_values = [t['name'] for t in expected_tags]

        assert_equal(len(tags), 2)
        assert_true(expected_tag_values)
        for t in [u'aa', u'aaa']:
            assert_true(t in tags)

        # has pagination
        assert_true(tag_response_html.select('.pagination'))

    def test_tag_search(self):
        '''Tag search returns expected results'''
        app = self._get_test_app()
        expected_tags = _make_tag_list(50)
        expected_tags.append({'name': 'find-me'})
        factories.Dataset(tags=expected_tags)

        tag_index_url = url_for(controller='tag', action='index')
        tag_response = app.get(tag_index_url)

        search_form = tag_response.forms[1]
        search_form['q'] = 'find-me'
        search_response = webtest_submit(search_form, status=200)

        search_response_html = BeautifulSoup(search_response.body)
        tags = [t.string for t in search_response_html.select('.tag')]

        assert_equal(len(tags), 1)
        assert_true('find-me' in tags)

        # no pagination
        assert_false(search_response_html.select('.pagination'))

    def test_tag_search_no_results(self):
        '''Searching for tags yielding no results'''
        app = self._get_test_app()
        expected_tags = _make_tag_list(50)
        factories.Dataset(tags=expected_tags)

        tag_index_url = url_for(controller='tag', action='index')
        tag_response = app.get(tag_index_url)

        search_form = tag_response.forms[1]
        search_form['q'] = 'find-me'
        search_response = webtest_submit(search_form, status=200)

        search_response_html = BeautifulSoup(search_response.body)
        tags = [t.string for t in search_response_html.select('.tag')]

        assert_equal(len(tags), 0)
        assert_true('find-me' not in tags)

        # no pagination
        assert_false(search_response_html.select('.pagination'))


class TestTagRead(helpers.FunctionalTestBase):

    def test_tag_read_redirects_to_dataset_search(self):
        app = self._get_test_app()
        factories.Dataset(title='My Other Dataset', tags=[{'name': 'find-me'}])

        tag_url = url_for(controller='tag', action='read', id='find-me')
        tag_response = app.get(tag_url, status=302)
        assert_equal(tag_response.headers['Location'],
                     'http://test.ckan.net/dataset?tags=find-me')

    def test_tag_read_not_found(self):
        '''Attempting access to non-existing tag returns a 404'''
        app = self._get_test_app()
        factories.Dataset(title='My Other Dataset', tags=[{'name': 'find-me'}])

        tag_url = url_for(controller='tag', action='read', id='not-here')
        app.get(tag_url, status=404)
